#include <unistd.h>
#include <sys/param.h>
#include <netinet/in.h>
#include "AgentApi.hxx"
#include "UdpStack.hxx"
#include "cpLog.h"

///
AgentApi::AgentApi(ServerType inSrvType) : ThreadIf()
{
//  case on the serverType, create datastructures for each type
//  open ports for ipc (defined ports)
//  if server, park a process on the port and have it enter the
//  callback on message reception.

    char hostname[MAXHOSTNAMELEN];
    srvType = inSrvType;
    switch(srvType) {
    case Agent :
	SockNum = AGENTTRAPPORT;
	MaxSockNum = SockNum;
	break;
    case RS :
	SockNum = RSPORT;
	MaxSockNum = SockNum;
	break;
    case NetMgnt :
	SockNum = MANAGERTRAPPORT;
	MaxSockNum = SockNum;
	break;
    case SipStack :
	SockNum = SIPSTACKPORT;
	MaxSockNum = MAXSIPSTACKPORT;
	break;
    default :
	SockNum = 0;
    }

    if (gethostname(hostname, MAXHOSTNAMELEN) < 0) {
	// todo.  probably want more of an error than this
	// maybe even kill the process.
	agentIpStr.assign("");
    } else 
	agentIpStr.assign(hostname);

    // this is the socket that the agent will recieve traps from the 
    // other processes and the processes will receive requests
    // from the agent.
    //
    // for now just do 1 stack.  in the future the agent will need many so 
    // that it can process multiple sessions.  These will be accessed
    // in  sendRequest.
    udpStack = new UdpStack(NULL, SockNum, MaxSockNum, sendrecv, false);
}

///
AgentApi::~AgentApi()
{
// close ports, deallocate memory
	cpLog( LOG_DEBUG, "entered agentApi destructor");
}

///
voReturnStatus
AgentApi::sendTrap(int trapType, string parameter)
{
    try {
	NetworkAddress dest(agentIpStr, AGENTTRAPPORT);
	memset(&trapMessage, 0, sizeof(trapMessage));
	trapMessage.action = (actionT)Trap;
	trapMessage.mibVariable = (AgentApiMibVarT)trapType;
	if (parameter.length() < PARM1SIZE ) {
	    strcpy(trapMessage.parm1, parameter.c_str());
	} else {
	    return voFailure;
	}
	udpStack->transmitTo((char *)&trapMessage, sizeof(trapMessage), &dest);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
}

voReturnStatus
AgentApi::sendResponse(int val, NetworkAddress *sender)
{
    try {
	int *parm2 = (int *)message.parm2;
	message.action = (actionT)Response;
	*parm2 = val;
	cpLog( LOG_DEBUG, "send integer response: %d to %s:%d", 
	       val, sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
    return voSuccess;
}

voReturnStatus
AgentApi::sendResponse(string parameter, NetworkAddress *sender)
{
    try {
	message.action = (actionT)Response;
	if (parameter.length() < PARM2SIZE ) {
	    strcpy(message.parm2, parameter.c_str());
	}
	else {
	    return voFailure;
	}
	cpLog( LOG_DEBUG, "send string response: %s to %s:%d", 
	       parameter.c_str(), sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
}

voReturnStatus
AgentApi::sendResponse(void *inData, NetworkAddress *sender)
{
    try {
	message.action = (actionT)Response;
	memcpy(message.parm2, inData, PARM2SIZE);
	cpLog( LOG_DEBUG, "send generic response to %s:%d", 
	       sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
}

voReturnStatus 
sendRequest(string indexName, string setValue)
{
    cpLog( LOG_ERR, "Using function not yet implemented 1" );
    return voFailure;
}

voReturnStatus 
sendRequest(string indexName, int setValue)
{
    cpLog( LOG_ERR, "Using function not yet implemented 2" );
    return voFailure;
}

void
AgentApi::thread() 
{
    int bytesRead = 0; 
    NetworkAddress sender;  

    ///  
    while (true)
    {
        try
        {
            bytesRead = udpStack->receiveFrom((char *)&message, sizeof(ipcMessage), &sender);
            cpLog( LOG_DEBUG, "bytes read=%d",bytesRead);  
            
	    processMessage(&message, &sender);

        }
        catch ( ... )
        {
            cpLog( LOG_ERR, "Exception from udpstack for AgentApiRx" );
        }
    }
}
