#ifndef AgentApi_H
#define AgentApi_H
//
//  This file is used by both the server (RS/FS...) C++ code and the snmp agent process
//  (snmpd ) C code.  The snmpd uses only the type definitions.

#ifdef __sparc
#define MAXHOSTNAMELEN 64
#endif 


#ifdef __cplusplus
#include "Sptr.hxx"
#include "Common.h"
#include "UdpStack.hxx"
#include "ThreadIf.hxx"
#include "cpLog.h"

#endif // __cplusplus
#include "AgentApiMibVars.hxx"

// global port settings will need more later
#define AGENTPORT 33600 
#define RSPORT    33601 
#define AGENTTRAPPORT 33602
#define MANAGERTRAPPORT 33603
#define MANAGERWEBRX    33604
#define MANAGERSTATUSPOLLER 33605
#define NWMUISOCKET 33606
#define SIPSTACKPORT 33607
#define MAXSIPSTACKPORT 33615

// global inter process communications timeouts in uSec
// put in file later
#define RSAGENTIPCTIMEOUT 3000000

// return value for statistics not yet implemented
#define VO_NA_INT -1
#define VO_NA_STRING "Not Implemented"

typedef enum
{
    Get = 1,
    Set,
    Trap,
    Response,
    Register,
} actionT;

#define PARM1SIZE 128
#define PARM2SIZE 1024
typedef struct 
{
    actionT action;
    AgentApiMibVarT mibVariable;
    ulong transactionNumber;
    char parm1[PARM1SIZE];
    char parm2[PARM2SIZE];
} ipcMessage;

#ifdef __cplusplus
extern "C" {

class AgentApi: public ThreadIf
{
public:
    ///
    AgentApi(ServerType inSrvType);
    ///
    virtual ~AgentApi();

    virtual voReturnStatus processMessage(ipcMessage *message, NetworkAddress *sender) = 0;

    // The following are generally used by the server processes
    voReturnStatus sendTrap(int trapType, string parameter);
    ///
    voReturnStatus sendResponse(int val, NetworkAddress *sender);
    voReturnStatus sendResponse(string parameter, NetworkAddress *sender);
    voReturnStatus sendResponse(void *inData, NetworkAddress *sender);
    
    //  The following are generally used by the agent processes
    voReturnStatus sendRequest(string indexName, string setValue);
    voReturnStatus sendRequest(string indexName, int setValue);

protected:
    ///
    virtual void thread();

private:
    ///
    Sptr<UdpStack> udpStack;
    int SockNum, MaxSockNum;
    ServerType srvType;
    ipcMessage trapMessage;
    ipcMessage message;
    string agentIpStr;
};

} // extern "C"
#endif //__cplusplus
#endif //AgentApi_H
