/*********************************************************************

 $Id: CommandLine.cxx,v 1.4 2000/05/25 23:33:40 bogawa Exp $

 *********************************************************************

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: CommandLine.cxx,v $
 Revision 1.4  2000/05/25 23:33:40  bogawa
 added LOG_DEBUG_STACK

 Revision 1.3  2000/05/24 22:00:26  ghuang
 Adding option LOG_HBDEBUG.

 Revision 1.2  2000/05/24 18:56:10  bogawa
 per-thread debugging for cpLog

 Revision 1.1  2000/05/11 01:47:59  ctam
 Creation.

 CommandLine.cxx is used to parse commandline arguments for the applications
 in the softswitch.

 Added Files:    CommandLine.cxx CommandLine.hxx


 ********************************************************************/

#include "CommandLine.hxx"
#include "cpLog.h"


///
Sptr<CommandLine> CommandLine::commandLine = 0;
///
const pair<const char*, const char*> CommandLine::cmdLineOptionString[] = { 
          pair<const char*, const char*>("pshostname", "ps.private.vovida.com"),
          pair<const char*, const char*>("psremoteport", "6005" ),
          pair<const char*, const char*>("sipport", "5060" ),
          pair<const char*, const char*>("retransmit", "0" ),
          pair<const char*, const char*>("daemon", "0" ),
          pair<const char*, const char*>("debuglevel","LOG_INFO" ) };
///
Sptr<CommandLine> 
CommandLine::instance( const int argc, 
                       const char** argv, 
                       const char* applicationUsage = " [-rd] [-v DEBUG_LEVEL] pserver:port sip-port")
{
    if (argc <= 1)
    {
        cerr << "Usage: " << argv[0] << " " << applicationUsage << endl;
        exit(-1);
    }

    if (commandLine == 0 )
    {
        commandLine = new CommandLine();
        assert( commandLine != 0 );
        commandLine->setDefaultValues();
        commandLine->parseCommandLine( argc, argv );
   } 
    return commandLine;
}

///
void
CommandLine::setDefaultValues()
{
        for (unsigned int i = 0; 
             i< sizeof(cmdLineOptionString)/sizeof(cmdLineOptionString[0]); 
             i++)
        {
            cmdLineOptionTable.insert(pair<string, string>( 
                CommandLine::cmdLineOptionString[i].first, 
                CommandLine::cmdLineOptionString[i].second ));
        }
}

///
void
CommandLine::parseCommandLine( const int argc, const char**argv )
{
    int option_index = 0;
    int arg_count = 0;

    int c = 0;

    static struct option long_options[] =
    {
        { CommandLine::cmdLineOptionString[0].first, 1, 0, 0},
        { CommandLine::cmdLineOptionString[1].first, 1, 0, 0},
        { CommandLine::cmdLineOptionString[2].first, 1, 0, 0},
        { CommandLine::cmdLineOptionString[3].first, 1, 0, 0},
        { CommandLine::cmdLineOptionString[4].first, 1, 0, 0},
        { CommandLine::cmdLineOptionString[5].first, 1, 0, 0},
    };



    while ( (c = getopt (const_cast<int> (argc), 
                         const_cast<char**> (argv), 
                         "drv:") ) != EOF)
    {
        switch (c)
        {
            case 0:
            {
                // If this option set a flag, do nothing else now
 
                if (long_options[option_index].flag != 0 )      
                break;


                TableIter i = cmdLineOptionTable.find(long_options[option_index].name);

                if (i != cmdLineOptionTable.end() )
                {
                    cmdLineOptionTable[long_options[option_index].name] = optarg;
                }
                else
                {
                    cpLog(LOG_DEBUG, 
                          "Skipping Command Line Option. Option %s Value Type %s",  
                          long_options[option_index].name, 
                          optarg);
                }
            }
            break;

            case 'd':
            {
                // daemon flag
                cmdLineOptionTable[long_options[4].name] = "1";
            }
            break;

            case 'r':
            {
                // retransmit flag
                cmdLineOptionTable[long_options[3].name] = "1";
            }
            break;

            case 'v':
            {
                // verbose (debug msg) flag

                if (strcmp(optarg, "LOG_DEBUG") == 0 ||
                    strcmp(optarg, "LOG_INFO")  == 0 ||
                    strcmp(optarg, "LOG_ERR")  == 0 ||
                    strcmp(optarg, "LOG_HBDEBUG")   == 0 ||
		    strcmp(optarg, "LOG_DEBUG_STACK") == 0)
                {
                    cmdLineOptionTable[long_options[5].name] = optarg;
                }
                else
                {
                    printf("Not right level:'%s'\n", optarg);
                    exit(-1);
                }

            }
            break;

            case '?': // getopt_long already printed an error message
            break;

            default:
                printf ("?? getopt returned character code 0%o ??\n", c);

            // abort ();
        }
    }

    if((argc-optind) < 2)
    {
        printf("Not enough arguments\n");
        exit(-1);
    } 

    if((argc-optind) > 2)
    {
        printf("Too many arguments\n");
        exit(-1);        
    }

    if (optind < argc)
    {
        // printf ("non-option ARGV-elements...\n");
        while (optind < argc)
        {
            if (arg_count == 0)
            {
                // first required arg must be the host name/port.
                const string& psName = argv[optind];
                string::size_type start;
                string::size_type myLen;

                start = psName.find(":");
                myLen = psName.length();

                // printf("host:%s\n", (psName.substr(0, start)).c_str());
                // printf("port:%s\n", (psName.substr(start+1, myLen)).c_str());


                // Set the provisioning server hostname info
                cmdLineOptionTable[long_options[0].name] =
                    (psName.substr(0, start)).c_str();

                // Check to see if the port info is given...
                if (start != -1 )
                {
                    // Overwrite default port info only if it's specified
                    cmdLineOptionTable[long_options[1].name] =
                        (psName.substr(start + 1, myLen)).c_str();
                }
                
            }
            else
            {
                // second required arg must be the sip port.

                cmdLineOptionTable[long_options[2].name] = argv[optind];

                // printf ("sipport:%s\n", argv[optind]);

            }

            arg_count++;           
            optind++;
        }
    }
}


///
Sptr<CommandLine>
CommandLine::instance()
{
    if (commandLine == 0 )
    {
        assert( commandLine != 0 );
        return 0;
    }
    else
    {
       return commandLine;
    }
}



///
const string&
CommandLine::getString(const string& cmdOption)
{
    static string nullstring = "";

    TableIter i = cmdLineOptionTable.find(cmdOption);

    if ( i != cmdLineOptionTable.end() )
        return i->second;

    return nullstring;
}

///
const int
CommandLine::getInt(const string& cmdOption)
{
    TableIter i = cmdLineOptionTable.find(cmdOption);

    int ret = 0;

    if ( i != cmdLineOptionTable.end() )
        ret = atoi(i->second.c_str());

    return ret;
}




// End of File
