#ifndef __vxworks

/*********************************************************************

 $Id: Connection.cxx,v 1.13 2000/05/24 03:04:29 surendra Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: Connection.cxx,v $
 Revision 1.13  2000/05/24 03:04:29  surendra

 Set the connection state alive in setState().

 Revision 1.12  2000/05/23 23:43:40  fhong
 Added static var to control the trapping of SIGPIPE, needed for when
 the connection is broken

 Revision 1.11  2000/05/10 21:56:58  bogawa
 fixes so that VxWorks compiles

 Revision 1.10  2000/05/11 01:50:01  surendra

 set the default waiting time for select and removed dataWaiting()

 Revision 1.9  2000/05/03 18:02:12  surendra
 Added assignment operator

 Revision 1.8  2000/04/26 17:34:48  surendra

 Made the accessor function constant so that it can build on SUN

 Revision 1.7  2000/04/12 00:28:47  fhong
 Added virtual destructor and bug fixes to dataWaiting()

 Revision 1.6  2000/04/06 19:47:55  fhong
 added method readn() with different signature and dataWaiting() which uses select

 Revision 1.5  2000/04/05 21:48:25  surendra

 Added option in the Connection class to make it blocking/non-blocking

 Revision 1.4  2000/04/03 17:02:07  surendra

 Made fixes so that it can compile on SUN.

 Revision 1.3  2000/03/07 05:58:14  surendra

 diabled the effRead() method for a while till I start using it in readn
 also

 Revision 1.2  2000/02/29 05:23:16  surendra

 Added copyright banner

 Revision 1.1  2000/02/29 05:16:49  surendra

 Moved class from external util to internal util.
 Revision 1.0  2000/02/17 surendra
     Initial Revision
 *********************************************************************/
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <string.h>
#include <fcntl.h>

#ifndef __vxworks
#include <signal.h>
#endif

#ifndef __vxworks
#include <sys/time.h>
#else
#include "vsock.hxx"
#endif

#include "VNetworkException.hxx"
#include "cpLog.h"
#include "Connection.hxx"

bool Connection::_init = false;

void
Connection::signalHandler(int signo)
{
    if(signo == SIGPIPE)
	cpLog(LOG_ALERT,"Caught SIGPIPE signal, broken pipe detected");
}

void
Connection::initialize()
{
    if(!_init)
    {

#ifndef __vxworks
	// setup SIGPIPE handler
	if(signal(SIGPIPE, signalHandler) == SIG_ERR)
	    cpLog(LOG_ALERT,
		  "Warning: Can't trap SIGPIPE, broken pipes will exit program");
#endif
	_init = true;
    }
}

int 
Connection::readLine(void* dataRead, size_t maxlen, int& bytesRead) throw (VNetworkException&)
{
    char c;
    int rc, i;
    char* ptr = (char*)dataRead;

    bytesRead = 0;
    for(i=1; i <= (int)maxlen; i++)
    {
        if((rc =effRead(&c)) == 1) {
            *ptr++ = c;
            bytesRead++;
            if(c == '\n') break;
        } else if(rc == 0) {
            if( i == 1) return 0;
            else break;
        } else {
            return -1;
        }
    }
    *ptr = 0;
    dataRead = ptr;
    return((i == 1) ? 0 : i);
}

int
Connection::readn(void *dataRead, size_t nchar, int & bRead) throw (VNetworkException&)
{
    size_t nleft;
    ssize_t nread;
    char *ptr = (char*)dataRead;

    nleft = nchar;

    while(nleft > 0)
    {
        if((nread  =read(_connId, ptr, nleft)) < 0) {
            if(errno == EINTR) nread = 0;
            else {
              return -1;
            }
        } else if(nread == 0) {
            //NO more data
            return 0;
        }
        nleft -= nread;
        ptr += nread;
        bRead += nread;
    }
    return(1);
}

int
Connection::readn(void *dataRead, size_t nchar) throw (VNetworkException&)
{
    if(dataRead == 0)
	return -1;

    return(read(_connId, static_cast<char*>(dataRead), nchar));
}

void 
Connection::writeData(string& data) throw (VNetworkException&)
{
   char *ptr = (char*)data.c_str();
   size_t nleft = data.size();
   writeData(ptr, nleft);
}

void 
Connection::writeData(void* data, size_t n) throw (VNetworkException&)
{
   size_t nleft;
   ssize_t nwritten;
   char *ptr = (char*)data;
   nleft = n;

   while(nleft > 0)
   {
       if((nwritten = ::write(getConnId(), ptr, nleft)) <= 0)
       {
           if(errno == EINTR) nwritten = 0;  //Write again
           else
           {
                 char buf[256];
                 sprintf(buf, "Failed to write data,reason:%s", strerror(errno));
                 cpLog(LOG_ALERT, buf);
                 throw VNetworkException(buf, __FILE__, __LINE__, errno);     
           }
       }
       nleft -= nwritten;
       ptr += nwritten;
   }
   //cpLog(LOG_DEBUG, "Bytes written:%d" , nwritten);
}

ssize_t
Connection::effRead(char* ptr)
{
    static int readCnt=0;
    static char* readPtr;
    static char readBuf[1];
    if(readCnt <= 0) 
    {
        while(1)
        {
            if((readCnt = ::read(getConnId(), readBuf, sizeof(readBuf))) < 0)
            {
                if(errno == EINTR){
                    continue;
                }
                return -1 ;
            } 
            else if(readCnt == 0)
            {
                return 0;
            }
            readPtr = readBuf;
            break;
        }
    }
    readCnt--;
    *ptr = *readPtr++;
    return 1;
}

string
Connection::getDescription() const
{
    string retStr;
    SA* sa = (SA*)&_connAddr;
    const u_char* p = (const u_char*)&_connAddr.sin_addr;
    switch(sa->sa_family) 
    {
        case AF_INET:
            char temp[256];
            snprintf(temp, sizeof(temp), "%d.%d.%d.%d",
                           p[0], p[1], p[2], p[3]);
            char pStr[56];
            sprintf(pStr, ":%d", ntohs(_connAddr.sin_port));
            retStr = temp;
            retStr += pStr;
            return (retStr);
        break;
        default:
            return "Unknown";
        break;
    }
    return ("Unknown");
}

void
Connection::close()
{
    ::close(_connId);
    _live = false;
    _connId = 0;
}

void
Connection::setState()
{
    if(!_blocking)
    {
        fcntl(_connId, F_SETFL, O_NONBLOCK);
    }
    _live = true;
}


bool
Connection::isReadReady(int seconds, int mSecconds) const
{
    fd_set rfds;
    struct timeval tv;
    int retval;

    //cpLog(LOG_DEBUG, "Checking connection %s", getDescription().c_str());
    FD_ZERO(&rfds);
    FD_SET(getConnId(), &rfds);
    tv.tv_sec = seconds;
    tv.tv_usec = mSecconds;
    int connId = getConnId();
    int maxFd = connId;
    retval = select(maxFd+1, &rfds, NULL, NULL, &tv);
    if (retval && FD_ISSET(connId, &rfds))
    {
        cpLog(LOG_DEBUG, "Data is ready on %s", getDescription().c_str());
        return true;
    }
    return false;
}

#endif
