/*********************************************************************

 $Id: Connection.hxx,v 1.9 2000/05/23 23:43:40 fhong Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: Connection.hxx,v $
 Revision 1.9  2000/05/23 23:43:40  fhong
 Added static var to control the trapping of SIGPIPE, needed for when
 the connection is broken

 Revision 1.8  2000/05/11 01:50:01  surendra

 set the default waiting time for select and removed dataWaiting()

 Revision 1.7  2000/05/03 18:02:13  surendra
 Added assignment operator

 Revision 1.6  2000/04/26 17:34:48  surendra

 Made the accessor function constant so that it can build on SUN

 Revision 1.5  2000/04/12 00:28:47  fhong
 Added virtual destructor and bug fixes to dataWaiting()

 Revision 1.4  2000/04/06 19:47:55  fhong
 added method readn() with different signature and dataWaiting() which uses select

 Revision 1.3  2000/04/05 21:48:25  surendra

 Added option in the Connection class to make it blocking/non-blocking

 Revision 1.2  2000/04/03 17:02:07  surendra

 Made fixes so that it can compile on SUN.

 Revision 1.1  2000/02/29 05:16:49  surendra

 Moved class from external util to internal util.
 ~

 Revision 1.5  2000/02/27 02:32:35  surendra

 Added copyright banner.

 Revision 1.4  2000/02/26 01:27:40  surendra

 Using string instead of mstring

 Revision 1.3  2000/02/26 00:46:47  surendra

 Added define for MAXLINE

 Revision 1.2  2000/02/25 04:54:44  surendra

 Changed the read iterface to read data from connection.

 Revision 1.1  2000/02/22 22:46:28  surendra

 Wrapper class for sockect connection.

 Revision 1.0  2000/02/17 surendra
     Initial Revision
 *********************************************************************/
#ifndef Connection_hxx
#define Connection_hxx

#include <netinet/in.h>
#include <string.h>

#include <string>

#ifdef __svr4__
    typedef int socklen_t;
#endif

//User define class

class VNetworkException;
class TcpServerSocket;
class TcpClientSocket;

typedef struct sockaddr SA;
#define MAXLINE   256

class Connection 
{
    public:
        Connection(bool blocking=true)
           : _connId(0),_live(false), _connAddrLen(0), _blocking(blocking)
	  {initialize();}

        Connection(int conId, bool blocking=true)
           : _connId(conId),_live(true), _connAddrLen(0), _blocking(blocking)
	  {initialize();}

        inline int getConnId() const { return _connId; } 
        inline struct sockaddr_in& getConnAddr() { return _connAddr; }
        inline socklen_t getConnAddrLen() const { return _connAddrLen; }

        Connection(const Connection& other) 
        {
            _connId = other._connId;
            _live   = other._live;
            _connAddrLen = other._connAddrLen;
            _blocking = other._blocking;
            memcpy((void*)&_connAddr, (void*)&other._connAddr, sizeof(_connAddr));
            setState();
        }

        virtual ~Connection()
	{
	    //if(_connId !=0)
		//close();
	}

        Connection& operator=(const Connection& other) 
        {
            if(this != &other)
            {
                _connId = other._connId;
                _live   = other._live;
                _connAddrLen = other._connAddrLen;
                _blocking = other._blocking;
                memcpy((void*)&_connAddr, (void*)&other._connAddr, sizeof(_connAddr));
                setState();
            }
            return *this;
        }

        bool operator==(const Connection& other)
        {
            return(_connId == other._connId);
        }

        bool operator!=(const Connection& other)
        {
            return(_connId != other._connId);
        }

        /**Reads line till '\n' is encountered or data ends.
           Returns 0 if no more data, else return number of bytes read
           Returns -1 if connection is closed
         */
        int readLine(void* data, size_t maxlen, int &bytesRead) throw (VNetworkException&);

        /**Reads nchar from the connection.
           Returns 0 if no more data, else return number of bytes read
           Returns -1 if connection is closed
         */
        int readn(void* data, size_t nchar, int &bytesRead) throw (VNetworkException&);

        int readn(void *data, size_t nchar) throw (VNetworkException&);

        ///Writes data to the connection
        void writeData(void* data, size_t n) throw (VNetworkException&);

        void writeData(string& data) throw (VNetworkException&);


        //Gets the connection description IP_ADDRESSS:Port
        string getDescription() const;

        /// Still connected?
        bool isLive() const {return(_live);}

        void close();

        ///Check if data is ready to be read within given seconds and microseconds
        bool isReadReady(int seconds=0, int mSecconds=20000) const;

        /// initialize the SIGPIPE signal handler (for broken pipes)
        void initialize();

        /// handler for SIGPIPE signal
        static void signalHandler(int signo);

    private:
       ssize_t effRead(char* ptr);

       /**Sets the connection state to be blocking or non-blocking
          based on the type of the connection.
        */
       void setState();

       friend class TcpServerSocket;
       friend class TcpClientSocket;
       int                _connId;
       bool               _live;
       socklen_t          _connAddrLen;
       struct sockaddr_in _connAddr;
       bool               _blocking;
       static bool        _init; /// Set to true if signal handler initialized
};

#endif
