/*********************************************************************

 $Id: ControlEvent.cxx,v 1.1 2000/05/27 07:14:36 icahoon Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: ControlEvent.cxx,v $
 Revision 1.1  2000/05/27 07:14:36  icahoon
 Event to control the starting, stopping and shutting down of a thread.

 ********************************************************************/


#include "ControlEvent.hxx"


VMutex	   ControlEvent::nextIdMutex;


int 	   ControlEvent::nextId = 0;


ControlEvent::ControlEvent(
    Type    	    	    	    	type_, 
    Fifo< Sptr<Event> >     	    * 	fifo_
)
    :	type(type_),
    	returnCode(-1),
	fifo(fifo_),
	id(0)
{
    nextIdMutex.lock();
    id = nextId++;
    nextIdMutex.unlock();    
}


ControlEvent::ControlEvent(
    Type    	    	    	    	type_, 
    Fifo< Sptr<Event> >     	    & 	fifo_
)
    :	type(type_),
    	returnCode(-1),
	fifo(&fifo_),
	id(0)
{
    nextIdMutex.lock();
    id = nextId++;
    nextIdMutex.unlock();    
}

ControlEvent::~ControlEvent()
{
}


const char* const   	
ControlEvent::name() const
{
    return ( "ControlEvent" );
}


ControlEvent::Type	    	    	    	
ControlEvent::getType() const
{
    return ( type );
}
		

void	    	    	    	
ControlEvent::setReturnCode(int rc)
{
    returnCode = rc;
}


int 	    	    	    	
ControlEvent::getReturnCode() const
{
    return ( returnCode );
}


void	    	    	    	
ControlEvent::setReplyFifo(
    Fifo< Sptr<Event> >     	&   fifo_
)
{
    fifo = &fifo_;
}


Fifo< Sptr<Event> > *
ControlEvent::getReplyFifo() const
{
    return ( fifo );
}


void	    	    	
ControlEvent::returnReply()
{
    if ( fifo != 0 )
    {
    	fifo->add(Sptr<Event>(this));
    }
}
		

int 	    	    	    	
ControlEvent::getControlEventId() const
{
    return ( id );
}


