/*********************************************************************

 $Id: Daemon.cxx,v 1.4 2000/04/03 23:25:31 ctam Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: Daemon.cxx,v $
 Revision 1.4  2000/04/03 23:25:31  ctam
 Add rules for vxworks compile.

 NOTE: functions UdpStack::receiveTimeout and Daemon are not compiled
       for vxworks.

 Modified Files:  Daemon.cxx UdpStack.cxx

 Revision 1.3  2000/04/02 16:20:56  eckelcu
 Added check for NO_DAEMON enviroment variable. Also added CVS headers
 and reindented.


 ********************************************************************/

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <iostream>
#include <stdlib.h>
#include "cpLog.h"
    
int Daemon()
{
#ifndef __vxworks
    pid_t pid;

    // start as daemon as default, but let environment variable override
    char* envValString = getenv( "NO_DAEMON" );
    if( envValString )
    {
	if( atoi( envValString ) != 0 )
	{
	    cerr << "NO_DAEMON environment variable set" << endl;
	    return 0 ;
	}
    }
    
    if( ( pid = fork() ) < 0 )
    {
	cerr << "first fork failed!" << endl;
	return  -1 ;
    }
    else if( pid != 0 )
    {
	exit ( 0 );
    }

    setsid();

    if( ( pid = fork() ) < 0 )
    {
	cerr << "second fork failed!" << endl;
	return -1 ;
    }
    else if ( pid != 0 )
    {
	exit( 0 );
    }

    setsid();

    umask( 0 );

    close( 0 );
    int fd;
    fd = open( "/dev/null", O_RDONLY );

    if( fd == -1 )
    {
	cerr << "reopen stdin failed!" << endl;
	return -1;
    }

    close( 1 );
    fd = open( "stdout.log", O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR );

    if( fd == -1 )
    {
	cerr << "reopen stdout failed!" << endl;
	return -1;
    }

    //  chmod("stdout.log", 0644);

    close( 2 );
    fd = dup( 1 );

    if( fd == -1 )
    {
	cerr << "reopen stderr failed!" << endl;
	return -1;
    }
    return 0;

#else
    cpLog(LOG_ERR, "Daemon  * not defined in vxworks *\n");

    return 0;
#endif
}
