/*********************************************************************

 $Id: Data.cxx,v 1.36 2000/05/26 21:28:06 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: Data.cxx,v $
 Revision 1.36  2000/05/26 21:28:06  bogawa
 fix for the Data to be part of the LOG_DEBUG_STACK

 Revision 1.35  2000/05/22 23:50:25  bogawa
 added non-case-sensitive fixes for Data

 Revision 1.34  2000/03/22 03:36:59  sunitha
 added #def values for FIRST, NOT_FOUND, FOUND , returned by the match function.
 Used by the parser. Also, changed match function to use this

 Revision 1.33  2000/02/24 17:48:18  eckelcu
 Added definition of pure virtual function getName() to DataException.
 Removed the files Connection.cxx, TcpClientSocket.cxx, and
 TcpServerSocket.cxx from Makefile because they do not compile for
 VxWorks and they need the licensing and logging information added to
 them.

 Revision 1.32  2000/02/18 01:43:28  sunitha
 modified match function to take the Data as pointer, and returning the errorcode

 Revision 1.31  2000/02/18 00:30:03  sunitha
 modified match to remove throwing exception

 Revision 1.30  2000/02/17 23:43:03  sunitha
 modified match function, so that, if the data object has
 sip:user@hostport, and we call match as:
 Data newdata = match(":", true);
 we get:  data = user@hostport, and newdata = sip.

 Revision 1.29  2000/02/17 23:06:12  sunitha
 changed match function to return the string upto the matched string.

 Revision 1.28  2000/02/15 19:06:01  sunitha
 added match function, which will be used by the sip parser.
 changed exception handling to use function from VException.
 tested match function.

 Revision 1.27  2000/01/31 20:20:47  sunitha
 made length function a const

 Revision 1.26  2000/01/25 01:57:26  bogawa
 const fixes for Data class

 Revision 1.25  2000/01/14 00:19:02  rag
 Data class fix -- make mutable stuff mutable

 Revision 1.24  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.23  2000/01/07 22:41:41  bogawa
 fixes for Data class

 Revision 1.22  1999/12/14 18:34:24  sunitha
 Removed comments

 Revision 1.21  1999/12/14 18:19:45  sunitha
 changes to == , and Data-> char, making these const functions

 Revision 1.20  1999/12/14 02:11:06  sunitha
 added charrep to Data class

 Revision 1.19  1999/12/14 01:50:41  sunitha
 Changed logic in conversion of Data to char*

 Revision 1.18  1999/12/14 01:21:35  sunitha
 Changed the Data to const char back to char.
 Changed getchar to getChar, since conflicting with macro definitions.

 Revision 1.17  1999/12/14 01:03:28  sunitha
 Changed getData to return const char.
 Changed the char* operator to be const char*

 Revision 1.16  1999/12/09 08:16:22  bogawa

 A bunch of mgcp fixes.  These files have been cleaned up a little.

 Revision 1.15  1999/12/04 23:49:54  sunitha
 fixed sprintf

 Revision 1.14  1999/12/04 22:18:03  cullen
 changed to use strings

 Revision 1.13  1999/12/04 06:45:20  cullen
 updated

 Revision 1.12  1999/12/04 04:18:05  tina
 *** empty log message ***

 Revision 1.11  1999/12/04 04:46:55  sunitha
 Support for int

 Revision 1.10  1999/12/04 04:12:58  sunitha
 contains support for mstring

 Revision 1.9  1999/12/04 02:42:06  sunitha
 added automatic type converters

 Revision 1.8  1999/12/03 19:06:17  cullen
 fiexed compare - lame

 Revision 1.7  1999/12/03 04:40:43  cullen
 might have fixed += and ==

 Revision 1.6  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

 Revision 1.5  1999/11/28 18:16:27  cullen
 moved OUT_OF_RANGE to be static scope
    
 ********************************************************************/   

//Authors: Sunitha Kumar, Cullen Jennings

#include <cstdio>
#include <ctype.h>

#include "cpLog.h"
#include "Data.hxx"

DataException::DataException( const string& msg,
                              const string& file,
                              const int line,
                              const int error = 0 )
 : VException( msg, file, line, error )
{
}

    
string
DataException::getName( void ) const
{
    return "DataException";
}


                              

Data::Data():
  buf()
{
}


Data::Data( const char* str, int length ):
  buf(str)
{
}


Data::Data( const char* str ):
  buf(str)
{
}


Data::Data( const string& str):
  buf(str)
{
}


Data::Data( const mstring& mstr):
  buf(mstr)
{
}


Data::Data( int value)
{
    char str[256];
    sprintf(str, "%d", value);
    buf = string(str);
}


Data::Data( const Data& data ):
  buf(data.buf)
{
}

	
Data& 
Data::operator=( const char* str )
{
  buf = string(str);

    return (*this);
}


Data& 
Data::operator=( const Data& data )
{
  buf = data.buf;

    return (*this);
}
	

const char*
Data::getData() const
{
    return buf.c_str();
}


#if 0	
void Data::substr( int start, int length, char* deststr ) const
{
    if ( ( start < 0 ) || (length < 1) )
    {
	cpLog(LOG_ERR, "Data: substr: incorrect start or length parameters");
	throw DataAccessError(OUT_OF_RANGE);
    }
    else 
    {
	char* str = buffer->getBuffer();
	for ( *deststr = *str; *str != '\0'; deststr++, str++, *deststr = *str );
        
	//assign the null char.
	*deststr = '\0';
    }
}
#endif

	
char 
Data::getChar(int i) const
{
    if (( i < 0 ) || (i > static_cast<int>(buf.size()) ) )
    {
	cpLog(LOG_ERR, "Data:getchar: i is out of range.");
	throw DataException(
                        "i is out of range",
                        __FILE__,
                        __LINE__
                        );
	return '\0';
    }

    return buf[i];
}

	
void Data::setchar(int i, char c)
{
    if (( i < 0) || (i > static_cast<int>(buf.size()) ) )
    {
	cpLog(LOG_ERR, "Data:setchar: i is out of range.");
        throw DataException(
                        "i is out of range",
                        __FILE__,
                        __LINE__
                        );

    }
    else
    {
      buf[i] = c;
    }
}


char 
Data::operator[](int i)
{
    char ch;
    ch = getChar(i);
    return ch;
}


int 
Data::length() const 
{
  return buf.size();
}

	
bool Data::operator==(const char* str) const
{
    return ( buf == string(str) );
}

	
bool Data::operator==( const Data& data) const
{
    return ( buf == data.buf );
}

	
bool Data::operator!=(const char* str)
{
    return ( buf != string(str));
}


bool Data::operator!=(const Data& data)
{
    return (  buf != data.buf );
}

	
bool
 Data::operator>(const Data& data) const 
{
    return ( buf > data.buf );
}

	
bool
 Data::operator<(const Data& data) const
{
   return ( buf < data.buf );
}

	
Data 
Data::operator+(const Data& data)
{
  return Data( string(buf + data.buf ));
}
	
	
Data 
Data::operator+(const char* str)
{
  return Data( string(buf + string(str) ));
}
	
	
void 
Data::operator+=(const Data& data)
{
  buf += data.buf;
}

	
void 
Data::operator+=(const char* str)
{
  buf += string(str);
}
	
        
Data::~Data()
{

}


Data::operator string() const
{
  return buf;
}


Data::operator const char*() const
{
  
  return buf.c_str();
}


Data::operator mstring() const
{
  return mstring(buf);
}


Data::operator int() const
{
    return atoi( buf.c_str() );
}

int  Data::match( Data match, Data* data, bool replace, Data replaceWith)
{
    //Data data;

    int pos = static_cast<int>(buf.find(match.getData()));
    
    if (pos == static_cast<int>(string::npos))
    {
        cpLog(LOG_DEBUG_STACK, "Match not found");
        pos = NOT_FOUND;
    }
    else
    {
       if (data)
       {
           (*data) = buf.substr(0, pos);
       }
       if (replace)
       {
           buf.replace(0, (pos+match.length()), replaceWith.getData());
       }
       else
       {
           
       }
       if ( (data->length() == 0) && (pos != ( (int) string::npos) ) )
       {
            //the match string is the first item.
            pos = -2;
            pos = FIRST;
       }
     
    }
    if ( (pos != NOT_FOUND) && (pos != FIRST) )
    {
	pos = FOUND;
    }
    return pos;
}


bool isEqualNoCase( const Data& left, const Data& right )
{
    string::iterator leftIter = left.buf.begin();
    string::iterator rightIter = right.buf.begin();

    while( (leftIter != left.buf.end()) && (rightIter != right.buf.end()) )
    {
	if(toupper(*leftIter) != toupper(*rightIter))
	{
	    return false;
	}
	++leftIter;
	++rightIter;
    }

    if( (leftIter != left.buf.end()) || (rightIter != right.buf.end()) )
    {
	// since both aren't the same length, they're not equal
	return false;
    }

    return true;
}
