#ifndef _DATA_H_
#define _DATA_H_

/*********************************************************************

 $Id: Data.hxx,v 1.29 2000/05/22 23:50:25 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: Data.hxx,v $
 Revision 1.29  2000/05/22 23:50:25  bogawa
 added non-case-sensitive fixes for Data

 Revision 1.28  2000/03/22 03:36:59  sunitha
 added #def values for FIRST, NOT_FOUND, FOUND , returned by the match function.
 Used by the parser. Also, changed match function to use this

 Revision 1.27  2000/02/24 17:48:18  eckelcu
 Added definition of pure virtual function getName() to DataException.
 Removed the files Connection.cxx, TcpClientSocket.cxx, and
 TcpServerSocket.cxx from Makefile because they do not compile for
 VxWorks and they need the licensing and logging information added to
 them.

 Revision 1.26  2000/02/18 01:43:28  sunitha
 modified match function to take the Data as pointer, and returning the errorcode

 Revision 1.25  2000/02/17 23:06:12  sunitha
 changed match function to return the string upto the matched string.

 Revision 1.24  2000/02/15 19:06:01  sunitha
 added match function, which will be used by the sip parser.
 changed exception handling to use function from VException.
 tested match function.

 Revision 1.23  2000/01/31 20:20:47  sunitha
 made length function a const

 Revision 1.22  2000/01/25 01:57:26  bogawa
 const fixes for Data class

 Revision 1.21  2000/01/14 00:19:02  rag
 Data class fix -- make mutable stuff mutable

 Revision 1.20  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.19  2000/01/07 22:41:41  bogawa
 fixes for Data class

 Revision 1.18  1999/12/14 18:34:24  sunitha
 Removed comments

 Revision 1.17  1999/12/14 18:19:45  sunitha
 changes to == , and Data-> char, making these const functions

 Revision 1.16  1999/12/14 02:11:06  sunitha
 added charrep to Data class

 Revision 1.15  1999/12/14 01:21:35  sunitha
 Changed the Data to const char back to char.
 Changed getchar to getChar, since conflicting with macro definitions.

 Revision 1.14  1999/12/14 01:03:28  sunitha
 Changed getData to return const char.
 Changed the char* operator to be const char*

 Revision 1.13  1999/12/09 08:16:22  bogawa

 A bunch of mgcp fixes.  These files have been cleaned up a little.

 Revision 1.12  1999/12/04 22:18:03  cullen
 changed to use strings

 Revision 1.11  1999/12/04 06:45:20  cullen
 updated

 Revision 1.10  1999/12/04 04:47:08  sunitha
 Support for int

 Revision 1.9  1999/12/04 03:26:18  cullen
 updated

 Revision 1.8  1999/12/04 04:13:12  sunitha
 Contains support for mstring

 Revision 1.7  1999/12/04 02:38:53  sunitha
 Added automatic type converters : Data to string, and Data to char*

 Revision 1.6  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

 Revision 1.5  1999/11/28 18:16:27  cullen
 moved OUT_OF_RANGE to be static scope
    
 ********************************************************************/


//Authors: Sunitha Kumar, Cullen Jennings

#include <cstring>
#include <string>
#include "mstring.hxx"
#include "VException.hxx"

#define NOT_FOUND -1
#define FIRST -2
#define FOUND 0

/// Exception handling class DataException
class DataException : protected VException
{
public:
    ///
    DataException( const string& msg,
                   const string& file,
                   const int line,
                   const int error = 0 );
    
    string getName( void ) const;
};





/** This will eventually implemented the copy on write, reference
    counted smart buffer scheme
*/

class DataBuffer;


/** This will eventually implemented the copy on write, reference
    counted smart buffer scheme
*/

class Data
{

public:
    // creation members.
    ///
    //Data( const string& str);
    ///
    Data( );
    ///
    Data( const char* str );
    ///
    Data( const char* buffer, int length );
    ///
    Data( const Data& data );
    ///
    Data( const string& str);
    ///
    Data( const mstring& mstr);
    ///
    Data( const int value);

    ///
    ~Data();

    ///
    bool operator>(const Data& ) const ;
    ///
    bool operator<(const Data& ) const;
    
    ///
    Data& operator=(const char* str);
    ///
    Data& operator=(const Data& data);
    ///
    //Data& operator=(const string& str);

    //access members
    ///
    //string getData() const;


    // TODO: should getData() return const char* ?
    ///
  const char* getData() const;

    ///
    //void substr( int start, int length , char* str) const;
    //char* substr( int start, int length ) const;
    ///
    char getChar( int i ) const; //return the i'th char of string.
    ///
    void setchar( int i, char c ); //write to the i'th char of string.
    ///
    char operator[]( int i ); //return the i'th char of string.
    ///
    int length() const;
    ///
    bool operator==( const char* str ) const;
    ///
    bool operator==( const Data& data ) const;
    ///
    //bool operator==(const string& second );
    ///
    bool operator!=( const char* str );
    ///
    bool operator!=( const Data& data );
    ///
    //bool operator!=(const string& str);
    ///
    Data operator+( const Data& data);
    ///
    Data operator+( const char* str);
    ///
    //Data& operator+(const Data& firstsrc, const string& secondsrc);
	
    //modification members
    ///
    void operator+=(const Data&);
    ///
    void operator+=(const char*);
    ///
    //Data& operator+=(const string&);
    ///


    // conversion operators
    ///
    operator string() const;
      
    ///
    operator const char*() const;

    ///
    operator mstring() const;

    ///
    operator int() const;
    //destruction 
   
    ///
    //returns the starting pos of match in this data item.
    // if replace is true, then, the matched string is replaced with the 
    //replaceWith string.
    //if match string is not found, then MATCH_NOT_FOUND exception is thrown.
    int Data::match( Data match, Data* data, bool replace = false, Data replaceWith = "");

    /// do a case-insensitive match

    friend bool isEqualNoCase( const Data& left, const Data& right );

private:
    mutable string buf;
    //DataBuffer* buf;
};

#endif 
