static const char version[] =
"$Id: Fifo.cxx,v 1.19 2000/06/01 00:56:31 hyu Exp $";

/*********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: Fifo.cxx,v $
 Revision 1.19  2000/06/01 00:56:31  hyu
 Resurrecting the dead versions ...

 Revision 1.17  2000/04/20 01:22:34  bogawa
 added another thread test

 Revision 1.16  2000/03/23 01:33:48  vincent
 Make nextEventID a static variable as there are multiple fifo objects that share the same queue and multiple events in the queue had the same eventId.

 Revision 1.15  2000/01/12 23:48:06  bogawa
 fixes for minor issues in the Sptr and Fifo code

 Revision 1.14  2000/01/12 02:04:19  wenqing
 Modify the operator- of VTime to return a signed number instead of unsigned

 Revision 1.13  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.12  1999/12/30 20:40:24  eckelcu
 Modified Files:
 	Fifo.cxx Fifo.h VTime.cxx VTime.hxx
 Cleaned up and added comments to VTime class,
 and modified Fifo to use VTime class.

 Revision 1.11  1999/12/23 00:06:46  ctam
 Changed assertion from:

 assert(!errcode)
 to
 assert(errcode >=0)

 Files:  Fifo.cxx

 Revision 1.10  1999/12/21 21:51:46  bogawa
 fixes for the great renaming

 Revision 1.9  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

    
 ********************************************************************/

#include <algorithm>
#include <cassert>
#include <errno.h>
#include <iostream>
#include <unistd.h>
#include "Fifo.h"

// TODO - add counting of number Fifo and remove thread when last Fifo deleted
// TODO - use signals to break the sleep and get rid of maxErrorInMs idea

FifoEventId FifoBase::nextEventID = 0;
const int  FifoBase::maxErrorInMs = 50;
VMutex* FifoBase::onceControl;
VMutex* FifoBase::eventsMutex;
list<FifoBase::FifoEvent>  FifoBase::events;

    
bool
operator < ( FifoBase::FifoEvent& lhs, FifoBase::FifoEvent& rhs)
{
   return ( lhs.time < rhs.time );
}
    
    
FifoBase::FifoBase()
  // nextEventID(1)       // nextEventId is now static
{
    eventsMutex = new VMutex;
    assert(eventsMutex);
    onceControl = new VMutex;
    assert(onceControl);
}


void*
FifoBase::timerThreadMain(void* )
{
   int error=0;
   
   while (true)
   {
      FifoEvent event;
      bool goodEvent=false;

      // poll period for new events on event list 
      int delayMs = maxErrorInMs;
      
      if (true) // find the next timed event 
      {
	 // lock the object 
	 error = eventsMutex->lock();	 
	 assert( !error );
	 
	 if ( events.size() > 0 ) 
	 {
	    event = events.front(); 
	    goodEvent = true;
	 }
	 
	 // unlock the object 
	 error = eventsMutex->unlock();
	 assert( !error );
      }
      
      if ( goodEvent )
      {
         delayMs = event.time - getVTime();
      }
      
      // cerr << "Do delay " << delayMs << endl;
      
      if ( delayMs > 0 ) // nanosleep for delayMs until time for this event
      {
	 // set the max delay in case another event gets added to the list
	 if( delayMs > maxErrorInMs ) 
	 {
	    delayMs = maxErrorInMs;
	 }
	 
	 usleep( delayMs );
      }
      else  // ok go and execute this event 
      {
	 FifoEvent msg;
	 bool goodMsg = false;
	 
	 // lock the object 
	 error = eventsMutex->lock();	 
	 assert( !error );
	 
	 // get an event off the event list 
	 if ( events.size() > 0 )
	 {
	    msg = events.front(); 
	    events.pop_front();
	    goodMsg = true;
	 }
	 
	 // unlock the object 
	 error = eventsMutex->unlock();
	 assert( !error );
	 
	 // put this event on the correct Fifo 
	 if ( goodMsg )
	 {
	    FifoBase* fifo = msg.fifo;
	    assert( fifo );
	    fifo->add( msg );
	 }
      }
   }
   
   return NULL;
}


void 
FifoBase::startTimerThread()
{
    static bool once = false;
    static VThread* timerThread = 0;

    onceControl->lock();
    if (!once)
    {
	int errorcode;

	timerThread = new VThread;

        errorcode = timerThread->spawn( FifoBase::timerThreadMain );
	assert(errorcode >=0);
	once = true;
    }
    onceControl->unlock();
}


FifoEventId
FifoBase::addEvent(FifoEvent& msg,const int msDelay)
{
   // set the fifo 
   msg.fifo = this;   
   msg.time = getVTime() + (const unsigned int)msDelay;
   
   // add the event to the event list ------------------------------
   
   // lock the object 
   int error = eventsMutex->lock();	 
   assert( !error );
   
   // get a new ID 
   FifoEventId id = nextEventID++;
   msg.eventId = id;

   // put the message on the FIFO 
   events.push_back( msg );
   
   // get the list into priority order 
   events.sort();
   
   // unlock the object 
   error = eventsMutex->unlock();
   assert( !error );
      
   // make sure the timer thread is running 
   FifoBase::startTimerThread();

   return id;
}


class HasEventId
{
private:
   FifoEventId id;
public:
  HasEventId( const FifoEventId& theId ) : id(theId) {}

  bool operator() ( const FifoBase::FifoEvent& event ) const { return event.eventId == id; }
};


void
FifoBase::cancel( FifoEventId id )
{
  // lock the object 
  int error = eventsMutex->lock();	 
  assert( !error );

  // find the message and delete it 
  list<FifoBase::FifoEvent>::iterator p;
  p = find_if( events.begin(), events.end(), HasEventId(id) );
  if ( p != events.end() ) // if we found something, erase it 
  {
      // delete the underlying msg contained in the FifoEvent
      FifoEvent baseMsg = static_cast<FifoEvent>( *p );
      FifoBase* fifo = baseMsg.fifo;
      assert( fifo );
      fifo->deleteMsg( baseMsg );
      events.erase( p );
  }

  // unlock the object 
  error = eventsMutex->unlock();
  assert( !error );
}
