/*********************************************************************

 $Id: HeartbeatTxThread.cxx,v 1.3 2000/05/27 22:31:42 icahoon Exp $

 *********************************************************************

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: HeartbeatTxThread.cxx,v $
 Revision 1.3  2000/05/27 22:31:42  icahoon
 removed Fifo, using a mutexed boolean instead

 Revision 1.2  2000/05/27 07:10:02  icahoon
 using control event

 Revision 1.1  2000/05/27 05:46:18  icahoon
 Moved heartbeat thread from sipProxy base to util. Added the ability to
 shutdown the thread by using a fifo. Now delaying by using the fifo's
 addDelayMsg and getNext rather that usleep.

 Revision 1.9  2000/04/24 22:31:55  ghuang
 comment out "heartbeat sent".

 Revision 1.8  2000/04/17 18:03:40  ghuang
 increase HB inteval to 250ms, and turn on HB sent cpLogs.

 Revision 1.7  2000/04/12 01:22:25  ghuang
 comment out "sleep" & "heartbeat sent" statements.

 Revision 1.6  2000/04/12 01:17:23  ghuang
 change Heartbeat interval to send HB every second instead of
 every 10 seconds.

 Revision 1.5  2000/04/06 03:59:47  rag
  Changed #include < to #include "

 Revision 1.4  2000/03/21 22:28:08  ghuang
 added sipPort param into HeartbeatTxThread ctor.

 Revision 1.3  2000/03/14 19:55:35  ghuang
 created data struct to include in heartbeat message.

 Revision 1.2  2000/03/03 00:48:11  marcl
 Included unistd.h

 Revision 1.1  2000/03/02 01:17:53  ghuang
 cleaned up heartbeat base code


 ********************************************************************/

#include "unistd.h"
#include "cpLog.h"
#include "HeartbeatTxThread.hxx"
#include "ControlEvent.hxx"


/// Define HeartbeatTxThread next
///
HeartbeatTxThread::HeartbeatTxThread(
    	const Sptr<UdpStack> 	    udpStack, 
	const int   	    	    sipPort, 
    	int 	    	    	    delayIntervalInMs = 250,
	const char *	    	    multicastIP = "224.0.0.100",
	u_int16_t   	    	    multicastPort = 9000
)
    :	multicastGroupAddr(multicastIP, multicastPort),
    	delayInMillisec(delayIntervalInMs),
    	itsUdpStack(udpStack),
	itsSipPort(sipPort),
	doShutdown(false)
{
    assert(itsUdpStack != 0 );
}


HeartbeatTxThread::~HeartbeatTxThread()
{
}

void    	    
HeartbeatTxThread::shutdown()
{
    doShutdownMutex.lock();
    doShutdown = true;
    doShutdownMutex.unlock();
}

void
HeartbeatTxThread::thread()
{
    cpLog( LOG_DEBUG, "Start heartbeat thread" );

    int delay = delayInMillisec * 1000;

    while ( true )
    {
    	doShutdownMutex.lock();

     	if ( doShutdown )
 	{
	    doShutdownMutex.unlock();
 	    return;
 	}

	doShutdownMutex.unlock();

    	usleep(delay);

        // Send heartbeat 
	//
        heartbeatData  data;  
        data.port = itsSipPort;
        data.load = 5; //test    

        itsUdpStack->transmitTo( (char*)&data, sizeof(heartbeatData), &multicastGroupAddr);  
    }
}

// End of File
