/*********************************************************************

 $Id: HeartbeatTxThread.hxx,v 1.4 2000/06/06 01:48:48 ghuang Exp $

 *********************************************************************

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: HeartbeatTxThread.hxx,v $
 Revision 1.4  2000/06/06 01:48:48  ghuang
 add getThreadPriority().

 Revision 1.3  2000/05/27 22:31:42  icahoon
 removed Fifo, using a mutexed boolean instead

 Revision 1.2  2000/05/27 07:10:02  icahoon
 using control event

 Revision 1.1  2000/05/27 05:46:18  icahoon
 Moved heartbeat thread from sipProxy base to util. Added the ability to
 shutdown the thread by using a fifo. Now delaying by using the fifo's
 addDelayMsg and getNext rather that usleep.

 Revision 1.5  2000/04/06 03:59:47  rag
  Changed #include < to #include "

 Revision 1.4  2000/03/21 22:28:09  ghuang
 added sipPort param into HeartbeatTxThread ctor.

 Revision 1.3  2000/03/14 19:55:35  ghuang
 created data struct to include in heartbeat message.

 Revision 1.2  2000/03/09 03:56:01  rag
 Fixed Id

 Revision 1.1  2000/03/02 01:17:53  ghuang
 cleaned up heartbeat base code


 ********************************************************************/

#if !defined(HeartbeatTxTHREAD_H)
#define HeartbeatTxTHREAD_H

#include <cassert>
#include "Sptr.hxx"
#include "ThreadIf.hxx"
#include "UdpStack.hxx"
#include "VMutex.h"
#include "Common.h"


/// HeartbeatTxThread is responsible for sending a regular hearbeat to
/// a multicast address. Note that the responsibility of the heartbeat
/// is to provide a hardware reliability check for the monitor. It is not
/// designed as a software reliability check since whatever inputs that 
/// crashed the first program will probably just crash the backup server as
/// well so there is not much point in trying to resolve software bugs 
/// via heartbeat.

class HeartbeatTxThread: public ThreadIf
{
public:
    /// Create a heartbeat transmitter thread.
    /// The heartbeat is sent via the udp stack. 
    /// The sipPort is the local sip port.
    /// 
    HeartbeatTxThread(    	
    	const Sptr<UdpStack> 	    udpStack, 
	const int   	    	    sipPort, 
    	int 	    	    	    delayIntervalInMs	= 250,
	const char *	    	    multicastIP     	= "224.0.0.100",
	u_int16_t   	    	    multicastPort   	= 9000
    );

    /// Destroy the heartbeat thread. It will attempt to shutdown the
    /// thread.
    ///
    virtual ~HeartbeatTxThread();

    /// Explicitly stop the heartbeat thread.
    ///
    void    	    shutdown();

    /// get Thread priority 
    int getThreadPriority() { return ThreadIf::itsThread.getPriority(); } 

protected:
    /// The new thread executes this method.
    ///
    virtual void    thread();

private:

    /// The multicast group we are sending the heatbeat to.
    ///
    NetworkAddress  	    	    	multicastGroupAddr;  

    /// The delay interval.
    ///
    int     	    	    	    	delayInMillisec;
    
    /// The transmitter.
    //
    Sptr<UdpStack>  	    	    	itsUdpStack;

    /// The local sip port.
    ///
    int     	    	    	    	itsSipPort;    

    bool    	    	    	    	doShutdown;
    VMutex  	    	    	    	doShutdownMutex;    

    /// Suppress copying
    ///
    HeartbeatTxThread(const HeartbeatTxThread& rhs);
    HeartbeatTxThread& operator=(const HeartbeatTxThread& rhs);
};

#endif // !defined(HeartbeatTxTHREAD_H)
