/*********************************************************************

 $Id: HostMatch.cxx,v 1.2 2000/05/11 00:32:58 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: HostMatch.cxx,v $
 Revision 1.2  2000/05/11 00:32:58  bogawa
 fixes for the HostMatch

 Revision 1.1  2000/05/11 00:02:08  bogawa
 added hostmatch code

 ********************************************************************/

#include <deque>
#include <string>
#include "HostMatch.hxx"
#include <netdb.h>
#include <iostream>
#include <set>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <algorithm>


class SimpleAddress
{
public:
    SimpleAddress(const char* addr);
    ostream& print(ostream& s) const;

    friend ostream& operator<< ( ostream& s, 
                                  const SimpleAddress& obj );

    friend bool operator<( const SimpleAddress& lhs , 
			   const SimpleAddress& rhs );

    friend bool operator==( const SimpleAddress& lhs , 
			    const SimpleAddress& rhs );


private:
    char myAddr[4];
};


bool operator<( const SimpleAddress& lhs , 
		const SimpleAddress& rhs )
{
    if (memcmp(lhs.myAddr, rhs.myAddr, 4) < 0)
    {
	return true;
    }
    else
    {
	return false;
    }
}


bool operator==( const SimpleAddress& lhs , 
		const SimpleAddress& rhs )
{
    if (memcmp(lhs.myAddr, rhs.myAddr, 4) == 0)
    {
	return true;
    }
    else
    {
	return false;
    }
}


ostream& operator<< ( ostream& s, const SimpleAddress& obj )
{
    return obj.print(s);
}


SimpleAddress::SimpleAddress(const char* addr)
{
    memcpy(myAddr, addr, 4);
}


ostream& SimpleAddress::print(ostream& s) const
{
    unsigned int x;
    memcpy(&x, myAddr, 4);
    in_addr y;
    y.s_addr = x;
    s << inet_ntoa(y);
    return s;
}


deque<SimpleAddress> getAddrList(const string& host)
{
    deque<SimpleAddress> list;

    struct hostent* ent;

    ent = gethostbyname(host.c_str());

    if(!ent)
    {
	return list;
    }

    char** addrPtr;

    addrPtr = ent->h_addr_list;

    while(addrPtr && *addrPtr)
    {
	list.push_back(SimpleAddress(*addrPtr));
	addrPtr++;
    }

    return list;
}

set<SimpleAddress> getAddrSet(const string& host)
{
    set<SimpleAddress> list;

    struct hostent* ent;

    ent = gethostbyname(host.c_str());

    if(!ent)
    {
	return list;
    }

    char** addrPtr;

    addrPtr = ent->h_addr_list;

    while(addrPtr && *addrPtr)
    {
	list.insert(SimpleAddress(*addrPtr));
	addrPtr++;
    }

    return list;
}


bool hostsEqual(const string& first, const string& second)
{
    set<SimpleAddress> firstSet;
    set<SimpleAddress> secondSet;

    firstSet = getAddrSet(first);
    secondSet = getAddrSet(second);

    if(firstSet.size() == 0)
    {
	return false;
    }

    return firstSet == secondSet;
}


bool hostsIntersect(const string& first, const string& second)
{
    set<SimpleAddress> firstSet;
    set<SimpleAddress> secondSet;

    firstSet = getAddrSet(first);
    secondSet = getAddrSet(second);

    deque<SimpleAddress> result;

    // set_intersection is part of <algorithm> .  It calculates the
    // intersection between firstSet and secondSet

    set_intersection(firstSet.begin(), firstSet.end(),
		     secondSet.begin(), secondSet.end(),
		     back_inserter(result));

    if(result.size() > 0)
    {
	return true;
    }
    else
    {
	return false;
    }
}
