/**********************************************************************/
#ifndef NETWORKADDRESS_H
#define NETWORKADDRESS_H

static const char* const NetworkAddress_h_fileVersion =
"$Id: NetworkAddress.h,v 1.20 2000/05/15 23:50:27 wenqing Exp $";

/*********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: NetworkAddress.h,v $
 Revision 1.20  2000/05/15 23:50:27  wenqing
 add one function for get sockaddr type of address

 Revision 1.19  2000/04/17 21:56:25  rag
 Added hashIpPort()

 Revision 1.18  2000/02/24 18:32:22  bogawa
 fixes for VxWorks Compile

 Revision 1.17  2000/02/24 02:18:35  hong
 mplemented at-most-once functionality/re-send on timeout in MGCP

 Revision 1.16  2000/02/02 01:54:08  hong
 Replacing MGCP stack's own UDP implementation with the UDP stack in util

 Revision 1.15  2000/01/31 20:21:56  wenqing
 update for stack mode and some misc items

 Revision 1.14  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.13  1999/12/24 02:29:46  tina
 Added consistent cvs header and LGPL license info.

 Revision 1.12  1999/12/23 04:57:47  bogawa
 rearrange some more MGCP files in preparation for finishing the Embedded Requests implementation

 Revision 1.11  1999/12/16 20:32:20  wenqing
 fix up some bugs while write test programs

 Revision 1.10  1999/12/15 19:34:49  wenqing
 adding comment

 Revision 1.9  1999/11/19 00:38:00  bogawa
 changes for the util directory as well as using the new vmutex, etc. classes

 Revision 1.8  1999/11/12 21:47:09  wenqing
 more changes according to cullen's change in networkAddress class

 Revision 1.7  1999/11/12 16:32:22  cullen
 fixed type name

 Revision 1.6  1999/11/12 16:26:23  cullen
 few changes from cullen

 Revision 1.5  1999/11/12 03:09:56  wenqing
 More complete update version. They are compiled.

 Revision 1.4  1999/11/11 18:33:11  wenqing
 This is the first version. Ask for comments.

 ********************************************************************/

#include "global.h"

#include <iostream.h>
#include <string>
#include "vtypes.h"
#include <netinet/in.h>

#define IPV4_LENGTH 4

class NetworkAddress
{
public:
    // The host is not defined yet. 
    NetworkAddress ( int port = -1 );    
    
    // The host hostName can in the same form at passed to setHostName
    NetworkAddress ( string hostName, int port = -1 );

    // takes IP or DNS with option ":port" syntax
    // it can be "lab1.vovida.com" or "192.168.4.5" or
    // "lab1.vovoda.com:80" or ...
    void setHostName ( const string& theAddress );

    // set port
    void setPort ( int port );

    // get host name, like "lab1.vovida.com"
    string getHostName () const;

    // get "192.168.4.5" kind of string
    string getIpName () const;

    // return a uint
    u_int32_t getIp4Address () const;

    // return sockaddr kind of address
    void getSockAddr(struct sockaddr & socka) const;
    //
    int getPort () const;

/*        
    /// fill buffer with IP address
    void addressToChar(char * pcAddress) const;
    /// get n'th byte of IP address
    unsigned char operator [] ( int nth) const;
*/

    /// compare two IP addresses
    friend bool operator < (const NetworkAddress & xAddress,
			    const NetworkAddress & xAddress );
    
    /// compare two IP addresses
    friend bool operator == (const NetworkAddress & xAddress,
			     const NetworkAddress & xAddress );

    /// convert to string - returns same as getHostName
    operator string() ;

    friend ostream & operator<< ( ostream & xStream, 
				  const NetworkAddress& rxObj );

    /// assignment operator =
    NetworkAddress& operator=( const NetworkAddress& x );

    ostream & print ( ostream & xStr ) const;

    /// hash function, uses last 2 bytes of ip address + 2 bytes of port
    /// the bits 15-0 of ipaddress (Bigendian) becomes the Most Significant Byte(MSB) of hashkey,
    /// bits 15-0 of port becomes LSB of hashkey.
    u_int32_t hashIpPort( );
   
    /// hash function, uses last 2 bytes of ip address + 2 bytes of port
    static u_int32_t hashIpPort( const string& theAddress, const string& port );

    /// hash function, uses last 2 bytes of ip address + 2 bytes of port
    static u_int32_t hashIpPort( const u_int32_t ipAddress, const int port );

private:
    /// IP address value
    char ipAddress[IPV4_LENGTH]; // it's in network byte order

    /// port value
    int aPort;
};


inline ostream & 
operator<< ( ostream & xStream, const NetworkAddress& rxObj )
{
    return rxObj.print ( xStream );
}


#endif // NETWORKADDRESS_H

