#ifndef __vxworks

/*********************************************************************

 $Id: TcpClientSocket.cxx,v 1.7 2000/05/24 03:05:15 surendra Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************
 $Log: TcpClientSocket.cxx,v $
 Revision 1.7  2000/05/24 03:05:15  surendra

 Set the connection state after successfull connection to the server.

 Revision 1.6  2000/05/18 21:24:01  bogawa
 fixes to allow for dynamic search for provisioning listener

 Revision 1.5  2000/05/10 21:56:58  bogawa
 fixes so that VxWorks compiles

 Revision 1.4  2000/04/25 23:29:34  surendra

 Given an option to optionally leave the connection open.

 Revision 1.3  2000/04/03 17:02:55  surendra

 Made changes so that it can compile on sun.

 Revision 1.2  2000/02/29 05:23:16  surendra

 Added copyright banner

 Revision 1.1  2000/02/29 05:15:47  surendra

 Moved class from external util to internal util.
 Revision 1.0  2000/02/17 surendra
     Initial Revision
 *********************************************************************/
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <sys/socket.h>
#include <strings.h>


#include "VNetworkException.hxx"
#include "TcpClientSocket.hxx"
#include "VEnvVar.hxx"
#include "NetworkAddress.h"
#include "cpLog.h"

TcpClientSocket::TcpClientSocket(const string& hostName, bool closeCon)
    :_hostName(hostName),
     _serverPort(-1),
     _closeCon(closeCon)
{
    initalize();
}

TcpClientSocket::TcpClientSocket(const string& hostName, int servPort, bool closeCon)
    :_hostName(hostName),
     _serverPort(servPort),
     _closeCon(closeCon)
{
    initalize();
}

TcpClientSocket::TcpClientSocket(const NetworkAddress& server, bool closeCon)
    :_closeCon(closeCon)
{
    _hostName = server.getHostName();
    _serverPort = server.getPort();
    if(_serverPort == -1 )
    {
        _serverPort = VEnvVar::VPS_PORT;
    }
    initalize();
}

TcpClientSocket::TcpClientSocket(const TcpClientSocket& other)
{
    _conn = other._conn;
}

TcpClientSocket& 
TcpClientSocket::operator=(TcpClientSocket& other)
{
    if(this != &other)
    {
        _conn = other._conn;
        _hostName = other._hostName;
        _serverPort = other._serverPort;
        _closeCon = other._closeCon;
    }
    return *this;
}


void
TcpClientSocket::initalize()
{
    _conn._connId = ::socket(AF_INET, SOCK_STREAM, 0);
    if(!_conn._connId)
    {
	char buf[256];
	sprintf(buf, "Failed to connect to server %s, reason %s", 
		_hostName.c_str(), strerror(errno));
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }

    bzero(&_conn._connAddr, sizeof(_conn._connAddr));
    _conn._connAddr.sin_family = AF_INET;
    NetworkAddress na(_hostName, _serverPort);
    _conn._connAddr.sin_addr.s_addr = inet_addr(na.getIpName().c_str());
    _conn._connAddr.sin_port = htons(na.getPort());
}

TcpClientSocket::~TcpClientSocket()
{
    close();
}


void 
TcpClientSocket::connect() throw (VNetworkException&)
{
   if(::connect(_conn._connId, 
		(SA*)&_conn._connAddr, 
		sizeof(_conn._connAddr)) < 0) 
   {
       char buf[256];
       sprintf(buf, "Failed to connect to server %s, reason %s", 
	       _hostName.c_str(), strerror(errno));
       cpLog(LOG_DEBUG, buf);
       throw VNetworkException(buf, __FILE__, __LINE__, errno);
   } 
   _conn.setState();
}

void
TcpClientSocket::close()
{
    if(_closeCon)
        _conn.close();
}

#endif
