#ifndef __vxworks

/*********************************************************************

 $Id: TcpServerSocket.cxx,v 1.8 2000/05/29 22:30:21 rag Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: TcpServerSocket.cxx,v $
 Revision 1.8  2000/05/29 22:30:21  rag
 Display Port number, when bind fails

 Revision 1.7  2000/05/22 23:29:54  bogawa
 fixes fix the TIME_WAIT problem for TcpServerSocket.cxx

 Revision 1.6  2000/05/18 21:24:02  bogawa
 fixes to allow for dynamic search for provisioning listener

 Revision 1.5  2000/05/10 21:56:58  bogawa
 fixes so that VxWorks compiles

 Revision 1.4  2000/04/05 21:48:25  surendra

 Added option in the Connection class to make it blocking/non-blocking

 Revision 1.3  2000/04/03 17:02:55  surendra

 Made changes so that it can compile on sun.

 Revision 1.2  2000/02/29 05:23:16  surendra

 Added copyright banner

 Revision 1.1  2000/02/29 05:17:02  surendra
 Moved class from external util to internal util.

 Revision 1.0  2000/02/17 surendra
     Initial Revision
 *********************************************************************/
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <strings.h>


#include "VNetworkException.hxx"
#include "TcpServerSocket.hxx"
#include "VEnvVar.hxx"
#include "cpLog.h"

#define LISTENQ   5
TcpServerSocket::TcpServerSocket(int servPort) throw (VNetworkException&)
{
    _serverConn._connId = ::socket(AF_INET, SOCK_STREAM, 0);

    if(_serverConn._connId < 0)
    {
	// this is an error to do something
	char buf[256];
	sprintf(buf, "socket failed, reason: %s", strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }

    int on = 1;

    if ( setsockopt ( _serverConn._connId, SOL_SOCKET, SO_REUSEADDR, 
		      &on, sizeof ( on )) )
    {
	// this is an error -- can't set it
	char buf[256];
	sprintf(buf, "setsockopt failed, reason: %s", strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }

    bzero(&_serverConn._connAddr, sizeof(_serverConn._connAddr));
    bzero(&_clientConn._connAddr, sizeof(_clientConn._connAddr));
    _serverConn._connAddr.sin_family = AF_INET;
    _serverConn._connAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    _serverConn._connAddr.sin_port = htons(servPort);
    if(::bind(_serverConn._connId, 
	      (SA*)&_serverConn._connAddr, 
	      sizeof(_serverConn._connAddr))
	) 
    {
	char buf[256];
	sprintf(buf, "Port %d, bind failed, reason:%s", servPort, strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }
    if(::listen(_serverConn._connId, LISTENQ))
    {
	char buf[256];
	sprintf(buf, "listen failed, reason:%s", strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }
}

TcpServerSocket::TcpServerSocket(const TcpServerSocket& other)
{
   _serverConn = other._serverConn;
   _clientConn = other._clientConn;
}

TcpServerSocket& 
TcpServerSocket::operator=(TcpServerSocket& other)
{

    if(this != &other)
    {
        _serverConn = other._serverConn;
        _clientConn = other._clientConn;
    }
    return *this;
}

TcpServerSocket::TcpServerSocket() throw (VNetworkException&)
{
    _serverConn._connId = ::socket(AF_INET, SOCK_STREAM, 0);

    if(_serverConn._connId < 0)
    {
	// this is an error to do something
	char buf[256];
	sprintf(buf, "socket failed, reason: %s", strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }

    int on = 1;

    if ( setsockopt ( _serverConn._connId, SOL_SOCKET, SO_REUSEADDR, 
		      &on, sizeof ( on )) )
    {
	// this is an error -- can't set it
	char buf[256];
	sprintf(buf, "setsockopt failed, reason: %s", strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }

    bzero(&_serverConn._connAddr, sizeof(_serverConn._connAddr));
    bzero(&_clientConn._connAddr, sizeof(_clientConn._connAddr));
    _serverConn._connAddr.sin_family = AF_INET;
    _serverConn._connAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    int port = VEnvVar::VPS_PORT;
    _serverConn._connAddr.sin_port = htons(port);

    if(::bind(_serverConn._connId, 
	      (SA*)&_serverConn._connAddr, 
	      sizeof(_serverConn._connAddr))
	) 
    {
	char buf[256];
	sprintf(buf, "Port %d, bind failed, reason:%s", port, strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }

    if(::listen(_serverConn._connId, LISTENQ))
    {
	char buf[256];
	sprintf(buf, "listen failed, reason:%s", strerror(errno));
	cpLog(LOG_ALERT, "%s", buf);
	throw VNetworkException(buf, __FILE__, __LINE__, errno);
    }
}

TcpServerSocket::~TcpServerSocket()
{
    close();
}


int 
TcpServerSocket::accept(Connection& con) throw (VNetworkException&)
{
   con._connAddrLen = sizeof(con._connAddr);
   if((con._connId = ::accept(_serverConn._connId, (SA*)&con._connAddr, &con._connAddrLen)) < 0) 
   {
       char buf[256];
       sprintf(buf, "Failed to except the connection, reason:%s", strerror(errno));
       cpLog(LOG_DEBUG, buf);
       throw VNetworkException(buf, __FILE__, __LINE__, errno);
   } 
   cpLog(LOG_DEBUG, "Connection from %s", con.getDescription().c_str());
   con._live = true;
   con.setState();
   return con._connId;
}

void
TcpServerSocket::close()
{
    ::close(_serverConn.getConnId());
}

#endif
