/*********************************************************************

 $Id: UdpStack.hxx,v 1.38 2000/03/28 22:42:39 rag Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: UdpStack.hxx,v $
 Revision 1.38  2000/03/28 22:42:39  rag
 Add receiveTimeout.  a receive call with a timeout based on select.

 Revision 1.37  2000/03/22 23:50:19  jeff
 added support for compilation on Sun Solaris

 Revision 1.36  2000/03/22 23:34:14  jeff
 *** empty log message ***

 Revision 1.35  2000/03/11 02:07:59  jeff
 change getDestinationHost() to return a ptr instead of instance.

 Revision 1.34  2000/02/24 01:37:55  wenqing
 add in select.h and #ifdef for sh-vxworks (selectLib.h)
 It is needed by fd_set*.

 Revision 1.33  2000/02/23 23:47:20  wenqing
 remove select.h include

 Revision 1.32  2000/02/15 03:43:10  wenqing
 adding files of inject udp pkts from log files.
 update for creating the log files.

 Revision 1.31  2000/02/11 22:13:51  wenqing
 adding handling for creating log files

 Revision 1.30  2000/02/02 02:13:01  hong
 Replacing MGCP stack' own UDP implementation with the UDP stack in util

 Revision 1.29  2000/01/31 20:21:56  wenqing
 update for stack mode and some misc items

 Revision 1.28  2000/01/27 20:58:33  wenqing
 update for modei(recvonly/sendonly/sendrecv) implementation

 Revision 1.27  2000/01/25 21:28:42  wenqing
 Update for add more checking for different mode(recvonly/sendrecv etc.)
 of the UdpStack.

 Revision 1.26  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.25  2000/01/07 22:21:40  wenqing
 Rename interface since it is reserved word for WIN32

 Revision 1.24  1999/12/24 03:07:34  tina
 rm #ifdef vxworks for making it to the OS independency.

 Revision 1.23  1999/12/24 00:33:04  wenqing
 fix for the problem of send/sento with using of connect()

 Revision 1.22  1999/12/22 01:37:23  wenqing
 add mode(recvonly, sendonly, sendrecv) to UdpStack

 Revision 1.21  1999/12/21 21:51:46  bogawa
 fixes for the great renaming

 Revision 1.20  1999/12/17 01:30:28  wenqing
 bug fixes

 Revision 1.19  1999/12/16 20:32:20  wenqing
 fix up some bugs while write test programs

 Revision 1.18  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

 Revision 1.17  1999/11/30 01:21:15  wenqing
 change xAddress(rxAddress and txAddress) to localAddr and remoteAddr.
 Add localPort and remotePort
 
 Revision 1.16  1999/11/25 00:36:37  wenqing
 add definitions for vxworks.

 Revision 1.15  1999/11/18 01:42:45  wenqing
 fix compiling error, very minor change.

 Revision 1.14  1999/11/18 00:32:35  wenqing
 Remove UdpPacket classes, and UdpErrorType enum. App can use Data class and
 UdpException class instead.

 Revision 1.13  1999/11/16 02:54:38  wenqing
 bug fixes

 Revision 1.12  1999/11/12 21:47:09  wenqing
 more changes according to cullen's change in networkAddress class

 Revision 1.11  1999/11/12 03:09:56  wenqing
 More complete update version. They are compiled.

 Revision 1.10  1999/11/11 03:01:32  wenqing
 This version is to continue to ask for comments

 Revision 1.9  1999/11/10 21:12:17  wenqing
 This is a new version of udp.hxx. New interfaces are added and it should be 
 a unified version for all applications.

 Revision 1.8  1999/10/25 22:02:20  bogawa
 merge from irix6-0914 back into the main branch

 Revision 1.7  1999/09/22 07:37:59  bogawa
 fixes for VON conference

 Revision 1.6.4.1  1999/09/15 00:24:21  bogawa
 merge in cheung's changes and changes to compile under sgi

 Revision 1.6  1999/08/17 21:03:45  kle
 Based on code from Bryan Ogawa, Cullen Jennings, & Kim Le
 Improvements on the jitter buffer.
 Flexability for 20ms on the wire.
 Added usage for Cullen's UDP code.
 Added usage for Chok's cpLog code.

 ********************************************************************/

#ifndef UDP_HXX
#define UDP_HXX

#include "global.h"
#include <fstream>
#include <string>
#include "NetworkAddress.h"
#ifdef __vxworks
 #include <selectLib.h>
#else
 #include <sys/select.h>
#endif

typedef enum
{
    inactive,
    sendonly,
    recvonly,
    sendrecv
} UdpMode;


// this class hold stuff that is likely to be a problem to port 
class UdpStackPrivateData;

/// classs implements a UDP transmitter AND a reciever 
class UdpStack
{
    /* Issues: Current no support for saying which interface a packet
       arrived on. I'm not sure why that woudl be usefull but I think
       someone requested it once.  */

   public:
    /** no ports can be specified and the stack will pick one, OR a minPort
        can be specified and it will be used, OR a min and max can be
        specified and the stack will use the first free port in that
        range. The destiantion hostname and port may be set in the
        destinationHost paramter **/
    UdpStack ( const NetworkAddress* destinationHost = NULL,
               int localMinPort = -1,
               int localMaxPort = -1,
               UdpMode mode = sendrecv,
               bool log_flag = false );

    /// connectPorts() has to be called before using send()
    /// It is associate the local port to remote port
    /// so that receive and send only from and to that remote port
    void connectPorts();

    /// disconnectPorts() dissolve the port association
    void disconnectPorts();

    /// set the local ports
    void setLocal (const int minPort = -1, int maxPort = -1 );

    /// set the default destination 
    void setDestination ( const NetworkAddress* host );

    /// set the default destination 
    void setDestination ( const char* host, int port );

    /// Get the udp stack mode 
    UdpMode getMode () { return mode; };

    /// Get the udp stack logFlag
    bool getLogFlag () { return logFlag; };

    /// Get the name for local
    string getLclName () const { return lclName; };

    /// Get the name for remote 
    string getRmtName () const { return rmtName; };

    /// Get the port being used for recieving  
    int  getRxPort () { return localPort; };

    /// Get the port being used for transmitting
    int  getTxPort () { return localPort; };

    /// Get the destination port 
    int  getDestinationPort () { return remotePort; };

    ///
    NetworkAddress* getDestinationHost () const;


    /// Set the mode
    void setMode ( const UdpMode theMode )
    { 
        mode = theMode; 
    };

    /// Set the logFlag
    void setLogFlag ( const bool theLogFlag )
    { 
        logFlag = theLogFlag; 
    };

    /// Set the name to something logical - only used for errors
    void setLclName ( const string& theName ) 
    { 
        lclName = theName; 
    };

    void setRmtName ( const string& theName ) 
    { 
        rmtName = theName; 
    };


    /// Get the file descriptor for the socket - use with care or not at all
    int  getSocketFD (); 

    /// Add this stacks file descriptors to the the fdSet
    void addToFdSet ( fd_set* set );

    /// Find the max of any file descripts in this stack and the passed value
    int  getMaxFD ( int prevMax = 0 );

    /// Check and see if this stacks file descriptor is set in fd_set
    bool checkIfSet ( fd_set* set );


    /// Recive a datagram into the specifed buffer - returns size of mesg.
    /// Once connectPorts() is called, can't receive from other ports
    int  receive ( const char* buffer,
                   const int bufSize ); // returns bytes read 

    /// Recive a datagram into the specifed buffer - returns size of mesg. 
    /// Use receive if you do not need to be able to tell who this packet 
    /// came from
    int  receiveFrom ( const char* buffer,
                       const int bufSize,
                       NetworkAddress* sender ); // returns bytes read 

    /// Recive a datagram into the specifed buffer - returns size of mesg.
    /// if successfull, -1 for error and 0 for timeout.
    int receiveTimeout ( const char* buffer,
			 const int bufSize,
			 NetworkAddress* sender,
			 int sec = 0, 
			 int usec = 0);

    /// Transmit a datagram
    /// ConnectPorts() should be called before it is called
    void transmit ( const char* buffer,
                    const int length );

    /// Transmit a datagram - use transmit if you alway sent to the same person
    void transmitTo ( const char* buffer,
                      const int length,
                      const NetworkAddress* dest );
    ///
    void joinMulticastGroup ( NetworkAddress group,
                              NetworkAddress* iface = NULL,
                              int ifaceInexe = 0 );
    ///
    void leaveMulticastGroup( NetworkAddress group,
                              NetworkAddress* iface = NULL,
                              int ifaceInexe = 0 );
    ///
    virtual ~UdpStack ();

    /// Get the number of bytes the stack has processed
    int  getBytesTransmitted ();

    /// Get the number of datagrams the stack has processed 
    int  getPacketsTransmitted ();

    /// Get the number of bytes the stack has processed
    int  getBytesReceived ();

    /// Get the number of datagrams the stack has processed 
    int  getPacketsReceived ();

    /** This can be used to eulate network packet loss by throwing 
        out a certain percentage of the tranmitted packets */
    void emulatePacketLoss ( float probabilityOfLoss )
    { 
        packetLossProbability = probabilityOfLoss; 
    };

private:
    /// Do not use 
    UdpStack& operator= ( const UdpStack& x ) 
    { 
        assert ( 0 ); 
        return *this; 
    };

    /// Do not use 
    UdpStack( const UdpStack& ) 
    { 
        assert ( 0 ); 
    };

    void doServer ( int minPort,
                    int maxPort );

    void doClient ( const NetworkAddress* desName,
                    int iPort );

private:
    /// port the stack is currently using 
    int localPort;

    /// port the stack is currently using 
    int remotePort;

    /// name of receiver 
    string lclName;

    /// name of transmitter
    string rmtName;

    /// probability of a given packet being thrown out by the stack
    float packetLossProbability;

    /// number of bytes the stack has received
    int numBytesReceived;

    /// number of datagrams the stack has received
    int numPacketsReceived;

    /// number of bytes the stack has transimitted
    int numBytesTransmitted;

    /// number of datagrams the stack has transimitted
    int numPacketsTransmitted;

    /// Mode 
    UdpMode mode;

    /// flag for msg log
    bool logFlag;

    /// private data that will be a pain to port 
    UdpStackPrivateData* data;

    ofstream* in_log;
    ofstream* out_log;

    int rcvCount ;
    int sndCount ;
};


/// Class to hold all types of exceptions that occur in the UDP stack
class UdpStackException
{
public:
    ///
    UdpStackException ( const string& description )
        : desc(description) 
    {};

private:
    string desc;

    friend ostream& operator<< ( ostream& strm , const UdpStackException& e );
};

inline ostream& 
operator<< ( ostream& strm , const UdpStackException& e )
{
    strm << e.desc;
    return strm;
}

//e exception class for when a transmited packet is renused 
class UdpStackExceptionConectionRefused: public UdpStackException
{
public:
    ///
    UdpStackExceptionConectionRefused( const string& description )
        : UdpStackException(description) 
    {};
};


/// exception class for when all ports all already in use 
class UdpStackExceptionPortsInUse: public UdpStackException
{
public:
    ///
    UdpStackExceptionPortsInUse ( const string& description )
        : UdpStackException(description) 
    {};
};


/// exception class for when a bogus hostname is given 
class UdpStackExceptionBadHostname: public UdpStackException
{
public:
    ///
    UdpStackExceptionBadHostname( const string& description )
        : UdpStackException(description) 
    {};
};



#endif
