/*********************************************************************

 $Id: VCondition.h,v 1.7 2000/05/28 00:46:03 icahoon Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: VCondition.h,v $
 Revision 1.7  2000/05/28 00:46:03  icahoon
 added the ability to have a timed wait

 Revision 1.6  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.5  1999/12/24 02:10:12  tina
 Added consistent cvs header and LGPL license info. to all the above
 files in /util.

    
 ********************************************************************/

#ifndef VCONDITION_H_
#define VCONDITION_H_

#include "VThread.h"


/** Class VCondition 
 ** Defines a simple conditional variable class which
 ** allows threads to block until shared data changes state.
 ** A VCondition must always be associated with a
 ** VMutex to avoid the race condition where a thread prepares to
 ** wait on a condition variable and another thread signals the
 ** condition just before the first thread actually waits on it.
 */
class VCondition
{
public:
    
    /** Create an VCondition object initialized with operating system
     ** dependant defaults (if any). 
     */
    VCondition (void);

    /**
     ** Delete a VCondition object
     */
    ~VCondition (void);
    
    /**
     ** Block on the condition. Will return after the relativeTime, 
     ** specified in millseconds, has passed. If set to -1, the default,
     ** it will wait indefinately. Uses the specified mutex to synchronize 
     ** access to the condition. Returns 0, if successful, or an errorcode.
     */
    int wait (VMutex* mutex, int relativeTimeInMs = -1);

    /** 
     ** Signal one waiting thread.
     ** Returns 0, if successful, or an errorcode.
     */
    int signal (void);

    /**
     ** Returns the operating system dependent unique id of the condition.
     */
    const vcondition_t*
    getId (void);

private:
    /// prevent assignment and initialization
    void operator= (const VCondition &);
    VCondition (const VCondition &);

    mutable vcondition_t  myId;
};

#endif // V_CONDITION_H_
