/*********************************************************************

 $Id: VEnvVar.hxx,v 1.3 2000/03/22 19:59:33 surendra Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: VEnvVar.hxx,v $
 Revision 1.3  2000/03/22 19:59:33  surendra

 Added env. variable for LOG LEVEL

 Revision 1.2  2000/03/17 16:41:27  surendra

 use c_str() instead of data() for mstring object.

 Revision 1.1  2000/02/29 05:16:36  surendra

 Moved class from external util to internal util.
 ~

 Revision 1.1  2000/02/22 22:47:51  surendra

 Wrapper class to get the environment variables and set the default if variable is not set.

 Revision 1.0  2000/02/17 surendra
     Initial Revision
 *********************************************************************/

#ifndef VEnvVar_hxx
#define VEnvVar_hxx

#include "mstring.hxx"


class VEnvVar {
  public:
    /**
       Constructor to create EnvVariable object.If variable is 
       not set , defvalue is taken.
    */
    VEnvVar(const mstring& variable, const mstring& defvalue);

    /**
       Constructor to create EnvVariable object.If variable is 
       not set , marked as not set;
    */
    VEnvVar(const mstring& variable);

    bool isSet() { return _set; };

    const mstring& value() { return _value; }

    operator mstring() const { return _value; }
    operator const char*() const { return _value.c_str(); }
    operator int() const { return _value; }

    //Add the new Env Variables as wwe go along
    static VEnvVar VPS_PORT; 
    static VEnvVar VCONFIG_DIR; 
    static VEnvVar VLISTENER_PORT; 
    static VEnvVar VLOG_FILE_NAME; 
    static VEnvVar VPS_FILESYSTEM; 
    static VEnvVar VPS_SERVER; 
    static VEnvVar VLOG_LEVEL; 

    private:
        mstring _variable;
        mstring _value;
        bool    _set;
};
#endif
