/*********************************************************************

 $Id: VException.hxx,v 1.4 2000/04/20 23:31:39 icahoon Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: VException.hxx,v $
 Revision 1.4  2000/04/20 23:31:39  icahoon
 Changed access to member data to protected rather than private.

 Revision 1.3  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.2  1999/12/23 21:02:37  eckelcu
 	Makefile VException.cxx VException.hxx VThread.h
 	VThreadException.cxx vthread-linux.cxx vthread-vxworks.cxx
 	test/VThreadTest.cxx
 Added Files:
 	VConditionException.cxx VConditionException.hxx
 Additional error and exception handling in VThread class

 Revision 1.1  1999/12/14 00:31:38  eckelcu
 Modified Files:
 	Makefile vthread-linux.cxx vthread.h
 Added Files:
 	VException.cxx VException.hxx VThreadException.cxx
 	VThreadException.hxx test/Makefile test/VThreadTest.cxx
 Created base exception class, VException, and modified VThread class
 to make use of it.
 Also created a new test program which exercises the new VThread class.


 ********************************************************************/

#ifndef VEXCEPTION_HXX
#define VEXCEPTION_HXX

#include <exception>
#include <string>
    
class VException : public std::exception
{
protected:
    
    /** Creates an exception object, should be called with a
     ** descriptive msg, the filename and line number where the
     ** exception occured, and optionally an error code asscociated
     ** with the exception
     ** Note, the log() method handles the logging format, so
     ** blank spaces, newlines, etc. should NOT be included in the
     ** parameters to the constructor.
     */
    VException( const string& msg,
		const string& file,
		const int line,
		const int error = 0);
    
    /** Returns the predefined name of the exception
     */
    virtual string getName( void ) const = 0;
    
    /** Logs the exception including the error code (if any) as well
     ** as the filename and line number where the exception
     ** occurred.
     **
     ** The logging is formatted as follows:
     ** <msg> [:<error>] at <file>:<line> 
     */
    void log( void ) const;

public:
    
    /** Returns the predefined name and user supplied msg
     */
    string getDescription( void ) const;

protected:
    /// user supplied msg
    string message;
    
    /// file in which exception occurred
    string fileName;
    
    /// line number at which exception occurred
    int lineNumber;

    /// user supplied error code (optional)
    int errorCode; 
};

    
/// Generic exception classes
/// Note, class specific exception classes should be defined in 
/// separate <ClassName>Exception.[ch]xx files
class VExceptionMemory: protected VException
{
public:
    VExceptionMemory( const string& msg,
		       const string& file,
		       const int line,
		       const int error = 0 );

    string getName( void ) const;
};


#endif // VEXCEPTION_HXX
