/*********************************************************************

 $Id: VMutex.h,v 1.5 2000/01/11 00:56:00 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: VMutex.h,v $
 Revision 1.5  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.4  1999/12/24 02:10:12  tina
 Added consistent cvs header and LGPL license info. to all the above
 files in /util.

    
 ********************************************************************/

#ifndef VMUTEX_H_
#define VMUTEX_H_

#include "VThread.h"
    
/** 
 ** Class VMutex
 ** Defines a simple mutex class which provides
 ** 'fast' mutex functionality, meaning that if the calling thread
 ** already has the mutex locked, a call to lock will block the
 ** thread forever.
 */
class VMutex
{
public:

    /** Create a VMutex object initialized with operating system
     ** dependent defaults (if any).
     */
    VMutex (void);

    /**
     ** Delete a VMutex object
     */
    ~VMutex (void);
    
    /** 
     ** Lock the mutex. If the mutex is currently unlocked, it
     ** becomes locked and owned by the calling thread, and
     ** lock returns immediately. If the mutex is
     ** already locked by another thread, lock
     ** suspends the calling thread until the mutex is unlocked.
     ** If the mutex is already locked by the calling thread, the
     ** calling thread blocks forever.
     ** Returns 0, if successful, or an errorcode.
     */
    int
    lock (void);
    
    /** 
     ** Unlock the mutex.  The mutex is assumed to be locked and owned
     ** by the calling thread, meaning that a call to unlock always
     ** returns it to the unlocked state. 
     ** Returns 0, if successful, or an errorcode.
     */
    int
    unlock (void);

    /**
     ** Returns the operating system dependent unique id of the mutex.
     */
    vmutex_t*
    getId (void);
    
private:
    /// prevent assignment and initialization
    void
    operator= (const VMutex &);
    VMutex (const VMutex &);
    
    mutable vmutex_t myId;
};

#endif // V_MUTEX_H_
