/*********************************************************************

 $Id: VThreadPool.hxx,v 1.2 2000/05/24 17:48:56 surendra Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************
 $Log: VThreadPool.hxx,v $
 Revision 1.2  2000/05/24 17:48:56  surendra
 Moved destructor in the cxx from hxx.

 Revision 1.1  2000/05/10 23:05:36  surendra
 Moved from policy/common

 Revision 1.1  2000/05/04 02:01:55  surendra

 Added threadpool object


 *********************************************************************/
#ifndef  VThreadPool_hxx
#define VThreadPool_hxx

#include <pthread.h>
#include <deque>
#include <list>

#include "VThread.h"
#include "VFunctor.hxx"
#include "VThreadInitException.hxx"
#include "Sptr.hxx"

/** Most of the code taken from "Pthreads Programming from O'REILLY publication" */

class VThreadPool
{
    public:
        typedef list<Sptr<VThread> > ListOfThreads;
        ///
        VThreadPool(int numThreads=1);

        virtual ~VThreadPool(); 

        ///
        void addFunctor(const VFunctor& functor) throw(VThreadInitException&);

        ///if finsh flag is true , will wait till the entire work queue is done
        void shutdown(bool finish=true);

        const ListOfThreads& getWorkerThreads() const { return _workerThreads; };

    private:
        ///
        void init()  throw (VThreadInitException&);

        ///
        static void* workerProc(void* arg); 

        deque<VFunctor> _queue;
        pthread_mutex_t  _queueLock;
        pthread_cond_t   _queueNotEmpty;
        pthread_cond_t   _queueNotFull;
        pthread_cond_t   _queueEmpty;
        int              _numThreads;
        int              _queueClosed;
        int              _shutdown;
        int              _currQueueSize;

        ListOfThreads   _workerThreads;
};

#endif
