/*********************************************************************

 $Id: VTime.hxx,v 1.6 2000/01/13 01:56:22 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: VTime.hxx,v $
 Revision 1.6  2000/01/13 01:56:22  bogawa
 VTime.cxx

 Revision 1.5  2000/01/12 02:04:19  wenqing
 Modify the operator- of VTime to return a signed number instead of unsigned

 Revision 1.4  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.3  1999/12/30 20:40:24  eckelcu
 Modified Files:
 	Fifo.cxx Fifo.h VTime.cxx VTime.hxx
 Cleaned up and added comments to VTime class,
 and modified Fifo to use VTime class.

    
 ********************************************************************/

#ifndef VTIME_H_
#define VTIME_H_

#ifdef __vxworks

#include <time.h>
#include <sys/times.h> 
    
/** 
 ** Operates the same as linux gettimeofday() except the
 ** usec field of struct timeval is rounded to nearest msec
 ** (eg. usec = msec*1000)
 */ 
int
gettimeofday (struct timeval *tv, struct timezone *tz);

#else

#include <sys/time.h>
    
#endif
// __vxworks

    
#include <sys/types.h>
#include "vtypes.h"

/** Class VThread 
 ** Defines a time class which provides means of
 ** creating, adding, subtracting, and comparing times. 
 ** VTime objects contain a second component and a fraction of second
 ** component. Both may be specified and accessed by the user.
 ** Routines to add and subtract times in terms of milliseconds are
 ** also provided.
 **/
class VTime
{
public:
    /// Create VTime object with specified seconds and fraction of a second.
    VTime( u_int32_t sec, u_int32_t frac ):seconds(sec),fractional(frac) {};
    
    /// Create a new VTime object with the current system time.    
    VTime() { *this = getVTime(); };
    
    /// Access functions
    u_int32_t getSeconds() { return seconds; };
    u_int32_t getFractional() { return fractional; };
    
    /// Display time stored in VTime in decimal notation 
    void print();
private:
    // this is the numbers of seconds
    u_int32_t seconds;
    /** this is the fraction of the second.  1 second == 2^32 fractional .
     */
    u_int32_t fractional;

    /// add specified number of milliseconds to VTime
    friend VTime operator+( const VTime& lhs , const unsigned int msec );

    /// subtract specified number of milliseconds from VTime
    friend VTime operator-( const VTime& lhs , const unsigned int msec );

    /// returns difference in terms of milliseconds, or 0 if rhs > lhs
    friend int operator-( const VTime& lhs , const VTime& rhs );

    friend bool operator==( const VTime& lhs , const VTime& rhs );
    friend bool operator<( const VTime& lhs , const VTime& rhs );
    friend bool operator>( const VTime& lhs , const VTime& rhs );

    /// returns a VTime object based on the current system time
    friend VTime getVTime();
};

#endif
// V_TIME_H_
