/*********************************************************************

 $Id: cpLog.cxx,v 1.15 2000/05/27 19:32:33 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: cpLog.cxx,v $
 Revision 1.15  2000/05/27 19:32:33  bogawa
 added cpLog fixes to do per-thread labeling

 Revision 1.14  2000/05/26 21:27:24  tmaneri
 removed parameter from cpLogGetPriority

 Revision 1.13  2000/05/26 17:46:14  bogawa
 fix for the util stuff

 Revision 1.12  2000/05/25 23:33:40  bogawa
 added LOG_DEBUG_STACK

 Revision 1.11  2000/05/25 03:54:41  bogawa
 make is so that file/line are no longer printed if they are nulled out

 Revision 1.10  2000/05/24 20:58:18  bogawa
 new log format for cplog

 Revision 1.9  2000/05/24 18:56:10  bogawa
 per-thread debugging for cpLog

 Revision 1.8  2000/05/11 17:19:09  icahoon
 flush to the file after each write

 Revision 1.7  2000/01/28 02:54:04  bogawa
 fixes for cpLog -- now prints file and line of cpLog message

 Revision 1.6  2000/01/11 00:56:00  bogawa
 copyright notice fixes for year 2000

 Revision 1.5  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

 Revision 1.4  1999/10/25 22:02:17  bogawa
 merge from irix6-0914 back into the main branch

 Revision 1.3  1999/10/11 01:30:52  cullen
 no real changes

 Revision 1.2.10.1  1999/09/15 00:57:11  bogawa
 merge in chok's new agent code

 Revision 1.2  1999/07/08 17:36:33  chok
 Merge DTMF digits changes from .../mgcp/example/agent.
 Change state engine function signature to be more flexible.

 ********************************************************************/

#include <sys/types.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <map>
#include "VThread.h"

#include "global.h"

#include "cpLog.h"
#include <string>

//#ifdef __cplusplus
//extern "C" {
//#endif

static char* priName[] =
{
    "EMERG",
    "ALERT",
    "CRIT",
    "ERR",
    "WARNING",
    "NOTICE",
    "INFO",
    "DEBUG",
    "DEBUG_STACK"
};


static char* priNameShort[] =
{
    "EMRG",
    "ALRT",
    "CRIT",
    "ERR ",
    "WARN",
    "NOTC",
    "INFO",
    "DBUG",
    "DSTK"
};

/* tbd: group the following variables into a struct */
//#define LOG_LABEL_MAX_SIZE 31
//static char cpLogLabel[LOG_LABEL_MAX_SIZE+1] = "cpLog";

//static int logPriority = LOG_ERR;
//static int logPriority = LOG_DEBUG;

#define LOG_FNAME_MAX_SIZE 256
static char cpLogFile[LOG_FNAME_MAX_SIZE+1] = "";

static FILE* cpLogFd = stderr;

class CpLogPriority
{
public:
    static int   getPriority();
    static void  setPriority(int pri);
    static void  setPriorityThread(vthread_t threadId, int pri);
    static void  clearPriorityThread(vthread_t threadId);

    static const char*  getLabel();
    static void         setLabel(const char* label);
    static void         setLabelThread(vthread_t threadId, const char* label);
    static void         clearLabelThread(vthread_t threadId);

protected:
    CpLogPriority();

private:
    static CpLogPriority* getInstance();
    static CpLogPriority* instance_;

    int logPriority;
    map < vthread_t, int > logPriorityMap;
    map < vthread_t, string > logLabelMap;
    string logLabel;
};


CpLogPriority* CpLogPriority::instance_ = 0;


CpLogPriority::CpLogPriority()
{
    logPriority = LOG_ERR;
}


CpLogPriority* CpLogPriority::getInstance()
{
    if(!instance_)
    {
	instance_ = new CpLogPriority;
    }
    return instance_;
}


int CpLogPriority::getPriority()
{
    map < vthread_t, int >::iterator i;

    i = getInstance()->logPriorityMap.find(VThread::selfId());

    if (i != getInstance()->logPriorityMap.end())
    {
	// found it!
	return i->second;
    }
    else
    {
	// not found, use default
	return getInstance()->logPriority;
    }
}


void 
CpLogPriority::setPriority(int pri)
{
    getInstance()->logPriority = pri;
}


void 
CpLogPriority::setPriorityThread(vthread_t threadId, int pri)
{
    getInstance()->logPriorityMap[threadId] = pri;
}


void
CpLogPriority::clearPriorityThread(vthread_t threadId)
{
    map < vthread_t, int >::iterator i;

    i = getInstance()->logPriorityMap.find(threadId);

    if (i != getInstance()->logPriorityMap.end())
    {
	getInstance()->logPriorityMap.erase(i);
    }
}


const char* CpLogPriority::getLabel()
{
    map < vthread_t, string >::iterator i;

    i = getInstance()->logLabelMap.find(VThread::selfId());

    if (i != getInstance()->logLabelMap.end())
    {
	// found it!
	return i->second.c_str();
    }
    else
    {
	// not found, use default
	return getInstance()->logLabel.c_str();
    }
}


void
CpLogPriority::setLabel(const char* label)
{
    getInstance()->logLabel = label;
}


void
CpLogPriority::setLabelThread(vthread_t threadId, const char* label)
{
    getInstance()->logLabelMap[threadId] = label;
}


void
CpLogPriority::clearLabelThread(vthread_t threadId)
{
    map < vthread_t, string >::iterator i;

    i = getInstance()->logLabelMap.find(threadId);

    if (i != getInstance()->logLabelMap.end())
    {
	getInstance()->logLabelMap.erase(i);
    }
}


bool
cpLogOpen (char* f)
{
    FILE* lfd;

    if ((lfd = fopen (f, "a+")) == (FILE*)0)
    {
        cpLog (LOG_ERR, "cpLog: Cannot open %s - %s",
                          f, strerror(errno));
        return false;
    }
    cpLog (LOG_INFO, "Open new log file - %s", f);
    strncpy (cpLogFile, f, LOG_FNAME_MAX_SIZE);
    cpLogFile[LOG_FNAME_MAX_SIZE] = 0;
    cpLogFd = lfd;
    return true;
}    /* cpLogOpen */

#ifndef __GNUC__
void
cpLog(int pri, const char *fmt, ...)
{
    va_list ap;

    if (pri <= CpLogPriority::getPriority())
    {
        va_start(ap, fmt);
        vCpLog(pri, 0, 0, fmt, ap);
        va_end(ap);
    }
}    /* cpLog */

#endif

void cpLog_impl_(int pri, const char* file, int line, const char* fmt, ...)
{
    va_list ap;

    if (pri <= CpLogPriority::getPriority())
    {
        va_start(ap, fmt);
        vCpLog(pri, file, line, fmt, ap);
        va_end(ap);
    }
}

void
vCpLog(int pri, const char* file, int line, const char* fmt, va_list ap)
{
    time_t now;

    (void)time(&now);

    /* Use printf() for now */

    char datebuf[256];

    strftime(datebuf, 256, "%y%m%d-%H%M%S", localtime(&now));

    struct timeval tv;
    struct timezone tz;

    gettimeofday(&tv, &tz);

    char msbuf[50];
    
    sprintf(msbuf, ".%3.3ld", (tv.tv_usec / 1000));

    strcat(datebuf, msbuf);

    /* check for whether or not file or line is null! */
    char linebuf[256];
    if((file == 0) || (line == 0))
    {
	strcpy(linebuf, "");
    }
    else
    {
	char filebuf[256];
	strcpy(filebuf, file);
	int x = strlen(filebuf) - 1;
	bool done = false;
	while(!done && (x >= 0))
	{
	    if(filebuf[x] == '.')
	    {
		filebuf[x] = '\0';
		done = true;
	    }
	    x--;
	}
	sprintf (linebuf, "%s:%d:", filebuf, line);
    }

    fprintf (cpLogFd, "%s [%5.5ld] %s %s: %.30s ",
	     datebuf,
	     VThread::selfId(),
	     priNameShort[pri],
	     CpLogPriority::getLabel(),
	     linebuf
	     );
    vfprintf (cpLogFd, fmt, ap);
    fprintf (cpLogFd, "\n");
    fflush(cpLogFd);
}    /* vCpLog */


void
cpLogSetPriority (int pri)
{
    CpLogPriority::setPriority(pri);
}    /* cpLogSetPriority */

int
cpLogGetPriority ()
{
    return CpLogPriority::getPriority();
}    /* cpLogSetPriority */


void
cpLogSetPriorityThread (vthread_t thread_id, int pri)
{
    CpLogPriority::setPriorityThread(thread_id, pri);
}


void
cpLogClearPriorityThread (vthread_t thread_id)
{
    CpLogPriority::clearPriorityThread(thread_id);
}


void
cpLogSetLabel (const char* label)
{
    CpLogPriority::setLabel(label);
//    strncpy (cpLogLabel, label, LOG_LABEL_MAX_SIZE);
//    cpLogLabel[LOG_LABEL_MAX_SIZE] = 0;
}    /* cpLogSetLabel */


void
cpLogSetLabelThread (vthread_t thread_id, const char* label)
{
    CpLogPriority::setLabelThread(thread_id, label);
}


void
cpLogClearLabelThread (vthread_t thread_id)
{
    CpLogPriority::clearLabelThread(thread_id);
}





void
cpLogShow (void)
{
    fprintf (stderr, "\tLabel    : %s\n", CpLogPriority::getLabel());
    fprintf (stderr, "\tPriority : %s\n", priName[CpLogPriority::getPriority()]);
    fprintf (stderr, "\tFile     : %s (cpLogFd = %d)\n", cpLogFile, fileno(cpLogFd));
}    /* cpLogShow */

//#ifdef __cplusplus
//}
//#endif
