/*********************************************************************

 $Id: cpLog.h,v 1.13 2000/05/27 19:32:33 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 ********************************************************************/

#ifndef CPLOG_H
#define CPLOG_H       

#define __const const

#include <stdarg.h>
#include "VThread.h"

#ifndef LOG_EMERG
#define LOG_EMERG       0       /* system is unusable */
#define LOG_ALERT       1       /* action must be taken immediately */
#define LOG_CRIT        2       /* critical conditions */
#define LOG_ERR         3       /* error conditions */
#define LOG_WARNING     4       /* warning conditions */
#define LOG_NOTICE      5       /* normal but significant condition */
#define LOG_INFO        6       /* informational */
#define LOG_DEBUG       7       /* debug-level messages */
#define LOG_DEBUG_STACK 8       /* stack debug-level messages */
#endif

/*
 * tbd:
 *      cpLogClose(),
 *      remote log server
 *      multiple logs
 *      cpLogSetMask ()
 *      change attributes on the fly -> separate tasks
 *      open a new file when the current one is reaching a limit
 *      what else?
 *      ...
 */

/* Generate a log message using FMT string and option arguments.  */

extern void cpLog_impl_ (int pri, const char* file, int line, const char* fmt, ...);
#ifdef __GNUC__
#define cpLog(priority__, fmt__, args__...) \
    cpLog_impl_ (priority__, __FILE__, __LINE__, fmt__ , ##args__)
#else
/* this is the regular definition */
extern void cpLog (int pri, const char *fmt, ...);
#endif

/* Generate a log message using FMT and using arguments pointed to by AP.  */
extern void vCpLog (int pri, const char* file, int line, const char *fmt, va_list ap);

extern void cpLogSetPriority (int pri);

extern int cpLogGetPriority ();

extern void cpLogSetPriorityThread (vthread_t thread_id, int pri);

extern void cpLogClearPriorityThread (vthread_t thread_id);

extern void cpLogSetLabelThread (vthread_t thread_id, const char* label);

extern void cpLogClearLabelThread (vthread_t thread_id);

extern void cpLogSetLabel (const char* label);

extern void cpLogShow (void);

extern bool cpLogOpen (char*);

/* CPLOG_H */
#endif
