#ifndef MD5String_HXX
#define MD5String_HXX 

/* *********************************************************************
   
   $Id: md5String.hxx,v 1.3 2000/05/30 23:01:57 wbreu Exp $
   
   *********************************************************************
   
   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not; write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.
   
   Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

   *********************************************************************
   
   $Log: md5String.hxx,v $
   Revision 1.3  2000/05/30 23:01:57  wbreu
   added comment on usage

   Revision 1.2  2000/05/29 23:43:49  wbreu
   ensure _md5String is exactly 16 bytes long.

   Revision 1.1  2000/05/29 21:55:17  wbreu
   added md5String.hxx to util


   **********************************************************************/

#include <string>
#include <vmd5.h>
#include <cpLog.h>
    
static const char* const md5StringVersion = "$Id: md5String.hxx,v 1.3 2000/05/30 23:01:57 wbreu Exp $";


/// C++ wrapper for vmd5.[ch]
//  Usage:
//  #include "md5String.hxx"
//
//   md5String  md5Str;
//   string     key = "E944D9A6-88C509CE-0-2888184C@192.168.6.210";
//   string     digest = md5Str.getMd5String( key );
//
class md5String 
{
public:
    ///
    md5String() { MD5Init( &_context ); }

    ///
    ~md5String() {} 

    // given a string, compute the message digest  
    const string& getMd5String( const string& buf ) {
 
        unsigned char digest[16];
        
        MD5Update( &_context, 
            reinterpret_cast<unsigned const char*>(buf.c_str()), 
            buf.length() );       

        MD5Final( digest, &_context );
        
        /// note: to make digest printable, need to encode digest
        ///       in base64 encoding
        _md5String = string(reinterpret_cast<char*>(digest), 16);
        return _md5String;
     }

///

private:

    MD5Context  _context;     
    string      _md5String;

    ///
    md5String( const md5String& rhs ) { assert( 0 ); }
    ///
    md5String& operator=( const md5String& rhs ) { assert( 0 ); }

};

#endif
