/*********************************************************************

 $Id: substring.h,v 1.7 2000/01/11 00:56:01 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: substring.h,v $
 Revision 1.7  2000/01/11 00:56:01  bogawa
 copyright notice fixes for year 2000

 Revision 1.6  2000/01/07 01:04:46  bogawa
 embedded request code integrated, including API support for these

 Revision 1.5  1999/12/24 02:10:12  tina
 Added consistent cvs header and LGPL license info. to all the above
 files in /util.

    
 ********************************************************************/

#ifndef SUBSTRING_H_
#define SUBSTRING_H_

#include <string>

class substring
{
public:
    substring() 
	: 
	myString(0),
	start(string::npos), 
	myLen(0)
	{
	}

    substring(const string& str)
	:
	myString(const_cast<string*>(&str)),
	start(0), 
	myLen(str.length())
	{
	}
	

    substring(const string* str, string::size_type i, string::size_type n) 
	: 
	myString(const_cast<string*>(str)),
	start(i), 
	myLen(n)
	{
	}

    substring(const substring& x)
	:
	myString(x.myString),
	start(x.start),
	myLen(x.myLen)
	{
	}

    substring(const substring& x, string::size_type i, string::size_type n)
	:
	myString(x.myString),
	start(x.start + i),
	myLen(n)
	{
	}

    substring& operator= (const substring& x)
	{
	    this->myString = x.myString;
	    this->start = x.start;
	    this->myLen = x.myLen;
	    return *this;
	}

    bool operator== (const string& x) const
	{
	    if(!myString)
	    {
		return false;
	    }
	    else
	    {
		return myString->substr(start, myLen) == x;
	    }
	}

    bool operator== (const char* x) const
	{
	    if(!myString)
	    {
		return false;
	    }
	    else
	    {
		return myString->substr(start, myLen) == x;
	    }
	}

    string str() const
	{
	    return myString->substr(start, myLen);
	}

    friend ostream& operator<<(ostream& s, const substring& x) {
	return s << x.str();
    }

    char operator[](string::size_type i) {
	return (*myString)[i + start];
    }

    const char operator[](string::size_type i) const {
	return (*myString)[i + start];
    }

    const char* c_str(void) const {
	return myString->substr(start, myLen).c_str();
    }

    string::size_type length() const {
	return myLen;
    }

    string::size_type getStart() const {
	return start;
    }

private:
    string* myString;
    string::size_type start;
    string::size_type myLen;
};

#endif
