/*********************************************************************

 $Id: udpTransport.hxx,v 1.16 2000/01/11 00:56:01 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: udpTransport.hxx,v $
 Revision 1.16  2000/01/11 00:56:01  bogawa
 copyright notice fixes for year 2000

 Revision 1.15  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

 Revision 1.14  1999/10/25 22:02:20  bogawa
 merge from irix6-0914 back into the main branch

 Revision 1.13.4.1  1999/09/30 00:51:05  bogawa
 fixes for new code

 Revision 1.13  1999/08/19 00:07:53  bogawa
 API change
  Modified Files:
  	Makefile commandCoding.cxx commandCoding.hxx eventCoding.cxx
  	eventCoding.hxx parameterCoding.cxx parameterCoding.hxx
  	sdp.cxx sdp.hxx support.cxx support.hxx udpTransport.cxx
  	udpTransport.hxx
  Added Files:
  	debug.cxx debug.h sptr.hxx

 Revision 1.12  1999/07/27 00:23:01  bogawa
 Add Win32 (Visual c++ 6) compilation support

 Revision 1.11  1999/07/23 23:57:25  bogawa
 the mgcp/examples/gateway Makefile was changed to use the new 
 make file system.
 improve the global makefiles.  Now, most targets use these makefiles
 which should simplify the lives of the users, we hope.
 Key features include support for nested object directories as well as
 a more coherent install strategy (so that installs are less likely to
 produce error messages)
 in addition, the code was cleaned up to add a few new features and to
 remove some warnings.

 Revision 1.10  1999/07/22 19:10:36  bogawa
 Updated API.  Major changes include threading changes and blocking API
 changes, also added new functionality.

 Revision 1.9  1999/07/09 08:20:04  bogawa
 Updated the code to do RequestedEvents properly.  Some stuff still
 doesn't work, but at least the Actions are now properly sent.
 This changes the exposed API for users, but the changes from
 here on out should be less.

 Revision 1.8  1999/06/30 06:09:08  bogawa
 update copyright and license

 Revision 1.7  1999/06/29 23:42:03  bogawa
 Updated copyright notice.

 Revision 1.6  1999/06/28 21:30:37  bogawa
 Update to the latest version of the MGCP stack.

 Revision 1.5  1999/06/09 08:16:45  bogawa
 Add copyright notices.

 Revision 1.4  1999/06/09 04:09:46  bogawa
 This makefile now correctly does diving makes.  In addition, the code
 has been cleaned up and adds some comments for doc++.  doc++ can be
 invoked by using make doc -- the current version will be shipped with
 appropriate html documentation.

 Revision 1.3  1999/06/09 01:04:59  bogawa
 Changes to make the new structure compile correctly, and to allow the
 code to use the new hardware structure.  gateway1 is now using the
 new structure fully, while gateway2 is using it partially (actually
 not very much).
 A future version should be much cleaner.

 Revision 1.2  1999/06/08 20:24:54  bogawa
 some more changes to allow for correct building and add the proper tags
 for cvs

 Revision 1.1.1.1  1999/06/08 18:12:25  bogawa
 make file system for cvs

 Revision 1.1.1.1  1999/06/04 23:52:54  bogawa
 create files
 Cullen Jennings, April 1999

 ********************************************************************/

#ifndef UDPTRANSPORT_HXX_
#define UDPTRANSPORT_HXX_






#include "global.h"
#include <sys/types.h>
#include <sys/select.h>
#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h> // for struct sockaddr_in
#endif

#include <vector>
#include <iostream>






///
typedef enum 
{
   MgcpUdpErrorNone=0,
   MgcpUdpErrorUnkownHost=100,
   MgcpUdpErrorConnectionLost,
   MgcpUdpErrorUndefined=-1
} ///
 MgcpUdpErrorType;


///
typedef void* MgcpUdpPacketPtr;

///
typedef void* MgcpUdpReceiverPtr;

///
typedef void* MgcpUdpTransmitterPtr;




/// class to hold a single RTP packet 
class MgcpUdpPacket
{
      // Curently header extentions are not supported

   public:
      /// Create a packet and allocate data space on heap
      MgcpUdpPacket(int payloadSize);
      
      /// Free space allocated on heap 
      ~MgcpUdpPacket();
      
      /// Get UDP messsage location
      char* getMessageLocation();
 
      /// Get UDP messsage length in bytes
      int getMessageSize();
 
      /// Set UDP messsage lenght in bytes
      void setMessageSize(int size);

      /// Print minimal info about packet, type, destination, timestamp
    // friend ostream& operator<<(ostream& s, const MgcpUdpPacket& x) {
    friend ostream& operator<<(ostream& s, MgcpUdpPacket& x) {
	x.print(s);
	return s;
    }

      /// Print minimal info about packet, type, destination, timestamp
      ostream& print( ostream& stream ) const;
      
      /// Print all info about a packet including hex version of payload data 
      void printVerbose( ostream& stream );
            
   private:
      /// pointer to the the raw data for the packet (including header)
      char* data;
      
      /// size of all the raw data for the packet (including header)
      int size;

      /// allocated size of data buffer 
      int allocatedSize;
};


///
class MgcpUdpStack
{
   public:
      /// 
      MgcpUdpStack(const char* serverName, int aPort );

   protected:
      ///
      int port;

      /// file descriptor of socket 
      int socketFd;
      
      /// address of computer that will receive the packets 
      struct sockaddr_in rxAddress;
      
      /// address of computer that is sending the packets
      struct sockaddr_in txAddress; 
};


///
class MgcpUdpReceiver: public MgcpUdpStack
{
   public:
      /// Create an input stack and start listening on aproperate port
      MgcpUdpReceiver( int port=5004 , int jitterBufferMs = 50 );

      /// Get the next chunk of data from the stream. Returns bytes received.
      int receive( char* buffer, int bufferSize);

      /// return a new UDP packet, non-blocking, NULL if none available, user must free this packet
      MgcpUdpPacket* receive(); 

      /// Set the stack so it generates an error if it received data from other hosts
      void onlyAcceptPacketsFrom( char* hostName );

      /// add this file descriptor to a FD_SET for a select call to use
      void addToFdSet( fd_set* set );
      
      /// Check if this file descriptor is set in an FD_SET for checking after a select call 
      bool checkIfSet( fd_set* set );
      int getFd() {
	  return socketFd;
      }
      
   private:
      /// read the next  packet off the network 
      MgcpUdpPacket* getPacket();

   private:
};


///
class MgcpUdpTransmitter: public MgcpUdpStack
{
   public:
      ///
      MgcpUdpTransmitter( const char* destinationName,
		      int port=5004 );

      /// transmit the data
      void transmit( char* data, int length );

      /// Stack will free this packet when it is done with it so it must be alloacted on the heap
      void transmit( MgcpUdpPacket* packet );

      /// will allocate a packet and fill in UDP header fields 
      MgcpUdpPacket* allocatePacket( int payloadLength );

    ///
    MgcpUdpTransmitter& operator=(const MgcpUdpTransmitter& x) {
	return *this;
    }

   public: // statistics
      ///
      int bytesTransmitted;
      
      ///
      int packetsTransmitted;
      
   private:
};


/// Class to hold all types of exceptions that occur in the UDP stack
class MgcpUdpException
{
   public:
      ///
      MgcpUdpException( MgcpUdpErrorType theErrorType );
      
      ///
      const MgcpUdpErrorType errorType;
};


#endif
