/*********************************************************************

 $Id: vcondition-linux.cxx,v 1.7 2000/05/28 03:03:47 icahoon Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: vcondition-linux.cxx,v $
 Revision 1.7  2000/05/28 03:03:47  icahoon
 fixed call to TIMEVAL_TO_TIMESPEC

 Revision 1.6  2000/05/28 00:46:03  icahoon
 added the ability to have a timed wait

 Revision 1.5  2000/01/11 00:56:01  bogawa
 copyright notice fixes for year 2000

 Revision 1.4  1999/12/21 21:51:46  bogawa
 fixes for the great renaming

 Revision 1.3  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

    
 ********************************************************************/

#ifndef __vxworks

#include <cassert>
#include <sys/time.h>
#include <unistd.h>
#include "VMutex.h"
#include "VCondition.h"
    
/*********************************************************************

VCondition for Linux

*********************************************************************/
    
VCondition::VCondition (void)
{
    // note, pthread implementation ignores the attribute variable, 
    // so it is set explicitly to NULL in the call to pthread_cond_init()
    int errorcode = pthread_cond_init(&myId, NULL);
    assert(!errorcode);
}
    
    
VCondition::~VCondition (void)
{
    int errorcode = pthread_cond_destroy(&myId);
    assert(!errorcode);
}

    
int
VCondition::wait (VMutex* mutex, int relativeTime)
{
    if ( relativeTime < 0 )
    {
    	return (pthread_cond_wait(&myId, mutex->getId()));
    }
    
    timeval 	expiresTV;
    int     	rc = gettimeofday(&expiresTV, 0);

    assert( rc == 0 );
    
    expiresTV.tv_usec += relativeTime / 1000;
    expiresTV.tv_usec += (relativeTime % 1000) * 1000;

    timespec	expiresTS;

    TIMEVAL_TO_TIMESPEC(&expiresTV, &expiresTS);

    return (pthread_cond_timedwait(&myId, mutex->getId(), &expiresTS));
}

    
int
VCondition::signal (void)
{
    return (pthread_cond_signal(&myId));
}


const vcondition_t*
VCondition::getId (void)
{
    return &myId;
}
    
#endif // not __vxworks
