/*********************************************************************

 $Id: vsock.cxx,v 1.6 2000/01/20 01:07:12 tina Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: vsock.cxx,v $
 Revision 1.6  2000/01/20 01:07:12  tina
 updated.

 Revision 1.5  2000/01/11 00:56:01  bogawa
 copyright notice fixes for year 2000

 Revision 1.4  2000/01/11 00:54:25  tina
 replace getuid() with getUser().

 Revision 1.3  1999/12/24 01:24:12  tina
 For OS independancy. Added consistent cvs header and LGPL license info.

 Revision 1.2  1999/12/24 00:55:20  tina
 Added consistent cvs header and LGPL license info.
    
 ********************************************************************/

#include "vsock.hxx"

#ifdef __vxworks
struct hostent* gethostbyname( const char* name )
{
    char buf[1024];
    return (hostent*)resolvGetHostByName( const_cast<char*>(name),
                                          buf, 1024 );
}

struct hostent* gethostbyaddr( const char* addr, int len, int type )
{
    char buf[1024]; // must be at least 512 bytes
    return (hostent*)resolvGetHostByAddr( addr, buf, 1024 );
}

unsigned int getpid( void )
{
    return (unsigned int)taskIdSelf();
}

#endif  // __vxworks


char* getUser( void )
{

#ifndef __vxworks
    static char user[64] = "unknown_user";
    uid_t uid = getuid();
    struct passwd *pw = getpwuid(uid);                  
    if (pw != NULL) 
    {
	strncpy(user, pw->pw_name, sizeof(user));
	user[63] = (char)0;
    }
    return (char*)user;
#else    
    return (char*)sysBootParams.usr;
#endif  //  __vxworks
}


#ifdef __vxworks
int strcasecmp(const char* str1, const char* str2)
{
    unsigned char res;

    while(1)
    {
        if( (res = (int)tolower(*str1) - (int)tolower(*str2++)) != 0 ||
        !*str1++ )
            break;
    }
    return res;
}


int strncasecmp(const char* str1, const char* str2, size_t len)
{
    unsigned char c1 = 0, c2 = 0;

    if( len )
    {
        do
        {
            c1 = *str1++; c2 = *str2++;
            if (!c1) break;
            if (!c2) break;
            if ( c1 == c2 ) continue;
            c1 = tolower( c1 );
            c2 = tolower( c2 );
            if ( c1 != c2 ) break;
        }while(--len);
    }

    return (int)c1 - (int)c2; 
}
#endif  //  __vxworks











