/*********************************************************************

 $Id: vthread-vxworks.cxx,v 1.12 2000/05/24 18:56:10 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: vthread-vxworks.cxx,v $
 Revision 1.12  2000/05/24 18:56:10  bogawa
 per-thread debugging for cpLog

 Revision 1.11  2000/04/20 23:30:17  icahoon
 cleaned up compile warningon selfId

 Revision 1.10  2000/04/20 01:22:34  bogawa
 added another thread test

 Revision 1.9  2000/01/11 00:56:01  bogawa
 copyright notice fixes for year 2000

 Revision 1.8  1999/12/30 23:58:06  eckelcu
 Modified Files:
 	VThread.h vthread-linux.cxx vthread-vxworks.cxx
 	test/VThreadTest.cxx
 Removed getHandle()  and fixed bug in exit() for linux.

 Revision 1.7  1999/12/23 21:02:37  eckelcu
 	Makefile VException.cxx VException.hxx VThread.h
 	VThreadException.cxx vthread-linux.cxx vthread-vxworks.cxx
 	test/VThreadTest.cxx
 Added Files:
 	VConditionException.cxx VConditionException.hxx
 Additional error and exception handling in VThread class

 Revision 1.6  1999/12/21 21:51:46  bogawa
 fixes for the great renaming

 Revision 1.5  1999/12/21 18:23:51  eckelcu

 Added usleep for vxworks

 Revision 1.4  1999/12/06 22:47:36  eckelcu
 Modified Files:
 	vthread-linux.cxx vthread-vxworks.cxx vthread.h
 Removed kill() method. Should use exit() instead.

 Revision 1.3  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

    
 ********************************************************************/

#ifdef __vxworks
    
#include <cassert>
#include "VThread.h"
#include "cpLog.h"

/*********************************************************************

VThread for VxWorks

*********************************************************************/

VThread::VThread( void )
{
    myId = VTHREAD_ID_INVALID;
}
    
VThread::~VThread( void )
{
    if( taskIdVerify( myId ) == OK )
    {
	// warn user but don't kill thread
	cpLog( LOG_DEBUG, "active VThread:%d going out of scope", myId );
    }
}
    
int
VThread::spawn( void *( *startFunc )( void* ),
		void *startArgs = 0,
		unsigned long flags = 0,
		unsigned long priority = VTHREAD_PRIORITY_DEFAULT,
		int stackSize = VTHREAD_STACK_SIZE_DEFAULT
    )
{
    if( myId != VTHREAD_ID_INVALID )
    {
	// make sure VThread doesn't already have an active thread
	// associated with it
	if( taskIdVerify( myId ) == OK )
	{
	    // return without spawning another thread
	    cpLog( LOG_ERR, "VThread:%d is already active", myId );
	    return VTHREAD_ID_INVALID;
	}
    }
    
    int vxworksPriority = VTHREAD_PRIORITY_VXWORKS_DEFAULT;
    int vxworksOptions = 0;

    assert( flags == 0 ); // handling tbd
    
    if( priority != VTHREAD_PRIORITY_DEFAULT )
    {
	vxworksPriority = priority;
    }

    myId = taskSpawn( 0, vxworksPriority, vxworksOptions, 
		      stackSize, (FUNCPTR) startFunc,
		      (int) startArgs,0,0,0,0,0,0,0,0,0 );
    if( myId <= 0 )
    {
	throw VExceptionMemory(
	    "cannot allocate VThread",
	    __FILE__,
	    __LINE__,
	    myId );
    }
    return 0;
}

    
int
VThread::join( void **status = 0 )
{
    while( taskIdVerify( myId ) == OK )
    {
	sleep( 1 );
    }
    return 0;
}
    

int
VThread::getPriority( void )
{
    int thePriority = VTHREAD_PRIORITY_DEFAULT;

    if( taskIdVerify( myId ) == 0 )
    {
	assert( taskPriorityGet( myId, &thePriority ) == OK );
    }
    else
    {
	cpLog( LOG_ERR, "VThread:%d is not active", myId );
    }
    return thePriority;
}


int
VThread::setPriority( int priority )
{
   return( taskPrioritySet( myId, priority ) );
}


const vthread_t*
VThread::getId( void )
{
    return &myId;
}

    
const vthread_attr_t*
VThread::getAttributes( void )
{
    return &myAttributes;
}


void
VThread::exit( void  )
{
    taskDelete( myId );
}
 

int
sleep( int seconds )
{
    return (int) (taskDelay( seconds * CLOCKS_PER_SEC ) );
}

int
usleep( int useconds )
{
    return (int) (taskDelay( ( useconds * CLOCKS_PER_SEC ) / 1000000 ) );
}

const vthread_t
VThread::selfId( void )
{
    return taskIdSelf();
}


#endif //__vxworks
